"""Factory functions for getting algorithms and problems by name."""

__all__ = ['get_algorithm', 'get_problem']


def get_problem(name, *args, **kwargs):
    r"""Get problem by name.

    Args:
        name (str): Name of the problem.

    Returns:
        Problem: An instance of Problem, instantiated with \*args and \*\*kwargs.

    Raises:
        KeyError: If an invalid name is provided.

    """
    problem = _problem_options().pop(name.lower())
    return problem(*args, **kwargs)


def get_algorithm(name, *args, **kwargs):
    r"""Get algorithm by name.

    Args:
        name (str): Name of the algorithm.

    Returns:
        Algorithm: An instance of the algorithm instantiated \*args and \*\*kwargs.

    Raises:
        KeyError: If an invalid name is provided.

    """
    algorithm = _algorithm_options().pop(name)
    return algorithm(*args, **kwargs)


def _problem_options():
    import niapy.problems as problems

    problems_dict = {
        'ackley': problems.Ackley,
        'alpine1': problems.Alpine1,
        'alpine2': problems.Alpine2,
        'bent_cigar': problems.BentCigar,
        'chung_reynolds': problems.ChungReynolds,
        'cosine_mixture': problems.CosineMixture,
        'csendes': problems.Csendes,
        'discus': problems.Discus,
        "dixon_price": problems.DixonPrice,
        "elliptic": problems.Elliptic,
        "conditioned_elliptic": problems.Elliptic,
        "expanded_griewank_plus_rosenbrock": problems.ExpandedGriewankPlusRosenbrock,
        "expanded_schaffer": problems.ExpandedSchaffer,
        "griewank": problems.Griewank,
        "happy_cat": problems.HappyCat,
        "hgbat": problems.HGBat,
        "katsuura": problems.Katsuura,
        "levy": problems.Levy,
        "michalewicz": problems.Michalewicz,
        "modified_schwefel": problems.ModifiedSchwefel,
        "perm": problems.Perm,
        "pinter": problems.Pinter,
        "powell": problems.Powell,
        "qing": problems.Qing,
        "quintic": problems.Quintic,
        "rastrigin": problems.Rastrigin,
        "ridge": problems.Ridge,
        "rosenbrock": problems.Rosenbrock,
        "salomon": problems.Salomon,
        "schaffer2": problems.SchafferN2,
        "schaffer4": problems.SchafferN4,
        "schumer_steiglitz": problems.SchumerSteiglitz,
        "schwefel": problems.Schwefel,
        "schwefel221": problems.Schwefel221,
        "schwefel222": problems.Schwefel222,
        "sphere": problems.Sphere,
        "sphere2": problems.Sphere2,
        "sphere3": problems.Sphere3,
        "step": problems.Step,
        "step2": problems.Step2,
        "step3": problems.Step3,
        "stepint": problems.Stepint,
        "styblinski_tang": problems.StyblinskiTang,
        "sum_squares": problems.SumSquares,
        "trid": problems.Trid,
        "weierstrass": problems.Weierstrass,
        "whitley": problems.Whitley,
        "zakharov": problems.Zakharov,
    }
    return problems_dict


def _algorithm_options():
    import niapy.algorithms.basic as basic_algorithms
    import niapy.algorithms.modified as modified_algorithms
    import niapy.algorithms.other as other_algorithms

    algorithms = {
        "BatAlgorithm": basic_algorithms.BatAlgorithm,
        "FireflyAlgorithm": basic_algorithms.FireflyAlgorithm,
        "DifferentialEvolution": basic_algorithms.DifferentialEvolution,
        "AgingNpDifferentialEvolution": basic_algorithms.AgingNpDifferentialEvolution,
        "DynNpDifferentialEvolution": basic_algorithms.DynNpDifferentialEvolution,
        "MultiStrategyDifferentialEvolution": basic_algorithms.MultiStrategyDifferentialEvolution,
        "DynNpMultiStrategyDifferentialEvolution": basic_algorithms.DynNpMultiStrategyDifferentialEvolution,
        "FlowerPollinationAlgorithm": basic_algorithms.FlowerPollinationAlgorithm,
        "GreyWolfOptimizer": basic_algorithms.GreyWolfOptimizer,
        "GeneticAlgorithm": basic_algorithms.GeneticAlgorithm,
        "ArtificialBeeColonyAlgorithm": basic_algorithms.ArtificialBeeColonyAlgorithm,
        "ParticleSwarmAlgorithm": basic_algorithms.ParticleSwarmAlgorithm,
        "ParticleSwarmOptimization": basic_algorithms.ParticleSwarmOptimization,
        "CenterParticleSwarmOptimization": basic_algorithms.CenterParticleSwarmOptimization,
        "MutatedParticleSwarmOptimization": basic_algorithms.MutatedParticleSwarmOptimization,
        "MutatedCenterParticleSwarmOptimization": basic_algorithms.MutatedCenterParticleSwarmOptimization,
        "ComprehensiveLearningParticleSwarmOptimizer": basic_algorithms.ComprehensiveLearningParticleSwarmOptimizer,
        "MutatedCenterUnifiedParticleSwarmOptimization": basic_algorithms.MutatedCenterUnifiedParticleSwarmOptimization,
        "OppositionVelocityClampingParticleSwarmOptimization": basic_algorithms.OppositionVelocityClampingParticleSwarmOptimization,
        "BareBonesFireworksAlgorithm": basic_algorithms.BareBonesFireworksAlgorithm,
        "CamelAlgorithm": basic_algorithms.CamelAlgorithm,
        "MonkeyKingEvolutionV1": basic_algorithms.MonkeyKingEvolutionV1,
        "MonkeyKingEvolutionV2": basic_algorithms.MonkeyKingEvolutionV2,
        "MonkeyKingEvolutionV3": basic_algorithms.MonkeyKingEvolutionV3,
        "EvolutionStrategy1p1": basic_algorithms.EvolutionStrategy1p1,
        "EvolutionStrategyMp1": basic_algorithms.EvolutionStrategyMp1,
        "EvolutionStrategyMpL": basic_algorithms.EvolutionStrategyMpL,
        "EvolutionStrategyML": basic_algorithms.EvolutionStrategyML,
        "SineCosineAlgorithm": basic_algorithms.SineCosineAlgorithm,
        "GlowwormSwarmOptimization": basic_algorithms.GlowwormSwarmOptimization,
        "GlowwormSwarmOptimizationV1": basic_algorithms.GlowwormSwarmOptimizationV1,
        "GlowwormSwarmOptimizationV2": basic_algorithms.GlowwormSwarmOptimizationV2,
        "GlowwormSwarmOptimizationV3": basic_algorithms.GlowwormSwarmOptimizationV3,
        "HarmonySearch": basic_algorithms.HarmonySearch,
        "HarmonySearchV1": basic_algorithms.HarmonySearchV1,
        "KrillHerd": basic_algorithms.KrillHerd,
        "FireworksAlgorithm": basic_algorithms.FireworksAlgorithm,
        "EnhancedFireworksAlgorithm": basic_algorithms.EnhancedFireworksAlgorithm,
        "DynamicFireworksAlgorithm": basic_algorithms.DynamicFireworksAlgorithm,
        "DynamicFireworksAlgorithmGauss": basic_algorithms.DynamicFireworksAlgorithmGauss,
        "GravitationalSearchAlgorithm": basic_algorithms.GravitationalSearchAlgorithm,
        "MothFlameOptimizer": basic_algorithms.MothFlameOptimizer,
        "FishSchoolSearch": basic_algorithms.FishSchoolSearch,
        "CuckooSearch": basic_algorithms.CuckooSearch,
        "CoralReefsOptimization": basic_algorithms.CoralReefsOptimization,
        "ForestOptimizationAlgorithm": basic_algorithms.ForestOptimizationAlgorithm,
        "MonarchButterflyOptimization": basic_algorithms.MonarchButterflyOptimization,
        "BacterialForagingOptimization": basic_algorithms.BacterialForagingOptimization,
        "BeesAlgorithm": basic_algorithms.BeesAlgorithm,
        "CatSwarmOptimization": basic_algorithms.CatSwarmOptimization,
        "HarrisHawksOptimization": basic_algorithms.HarrisHawksOptimization,
        "ClonalSelectionAlgorithm": basic_algorithms.ClonalSelectionAlgorithm,
        "LionOptimizationAlgorithm": basic_algorithms.LionOptimizationAlgorithm,
        "HybridBatAlgorithm": modified_algorithms.HybridBatAlgorithm,
        "AdaptiveBatAlgorithm": modified_algorithms.AdaptiveBatAlgorithm,
        "SelfAdaptiveBatAlgorithm": modified_algorithms.SelfAdaptiveBatAlgorithm,
        "HybridSelfAdaptiveBatAlgorithm": modified_algorithms.HybridSelfAdaptiveBatAlgorithm,
        "ParameterFreeBatAlgorithm": modified_algorithms.ParameterFreeBatAlgorithm,
        "DifferentialEvolutionMTS": modified_algorithms.DifferentialEvolutionMTS,
        "DifferentialEvolutionMTSv1": modified_algorithms.DifferentialEvolutionMTSv1,
        "DynNpDifferentialEvolutionMTS": modified_algorithms.DynNpDifferentialEvolutionMTS,
        "DynNpDifferentialEvolutionMTSv1": modified_algorithms.DynNpDifferentialEvolutionMTSv1,
        "MultiStrategyDifferentialEvolutionMTS": modified_algorithms.MultiStrategyDifferentialEvolutionMTS,
        "MultiStrategyDifferentialEvolutionMTSv1": modified_algorithms.MultiStrategyDifferentialEvolutionMTSv1,
        "DynNpMultiStrategyDifferentialEvolutionMTS": modified_algorithms.DynNpMultiStrategyDifferentialEvolutionMTS,
        "DynNpMultiStrategyDifferentialEvolutionMTSv1": modified_algorithms.DynNpMultiStrategyDifferentialEvolutionMTSv1,
        "SelfAdaptiveDifferentialEvolution": modified_algorithms.SelfAdaptiveDifferentialEvolution,
        "MultiStrategySelfAdaptiveDifferentialEvolution": modified_algorithms.MultiStrategySelfAdaptiveDifferentialEvolution,
        "NelderMeadMethod": other_algorithms.NelderMeadMethod,
        "HillClimbAlgorithm": other_algorithms.HillClimbAlgorithm,
        "SimulatedAnnealing": other_algorithms.SimulatedAnnealing,
        "MultipleTrajectorySearch": other_algorithms.MultipleTrajectorySearch,
        "MultipleTrajectorySearchV1": other_algorithms.MultipleTrajectorySearchV1,
        "AnarchicSocietyOptimization": other_algorithms.AnarchicSocietyOptimization,
        "RandomSearch": other_algorithms.RandomSearch,
    }
    return algorithms



"""Module with implementation of utility classes and functions."""

from niapy.util.argparser import get_argparser, get_args, get_args_dict
from niapy.util.array import full_array, objects_to_array
from niapy.util.distances import euclidean
from niapy.util.random import levy_flight
from niapy.util.repair import limit, limit_inverse, wang, rand, reflect

__all__ = [
    'get_argparser',
    'get_args',
    'get_args_dict',
    'full_array',
    'objects_to_array',
    'levy_flight',
    'euclidean',
    'limit',
    'limit_inverse',
    'wang',
    'rand',
    'reflect'
]



# encoding=utf8
"""Implementations of Weierstrass functions."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Weierstrass']


class Weierstrass(Problem):
    r"""Implementations of Weierstrass functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Weierstrass Function**

        :math:`f(\textbf{x}) = \sum_{i=1}^D \left( \sum_{k=0}^{k_{max}} a^k \cos\left( 2 \pi b^k ( x_i + 0.5) \right) \right) - D \sum_{k=0}^{k_{max}} a^k \cos \left( 2 \pi b^k \cdot 0.5 \right)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.
        Default value of a = 0.5, b = 3 and k_max = 20.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $$f(\textbf{x}) = \sum_{i=1}^D \left( \sum_{k=0}^{k_{max}} a^k \cos\left( 2 \pi b^k ( x_i + 0.5) \right) \right) - D \sum_{k=0}^{k_{max}} a^k \cos \left( 2 \pi b^k \cdot 0.5 \right)

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i=1}^D \left( \sum_{k=0}^{k_{max}} a^k \cos\left( 2 \pi b^k ( x_i + 0.5) \right) \right) - D \sum_{k=0}^{k_{max}} a^k \cos \left( 2 \pi b^k \cdot 0.5 \right) \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    def __init__(self, dimension=4, lower=-100.0, upper=100.0, a=0.5, b=3, k_max=20, *args, **kwargs):
        r"""Initialize Bent Cigar problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.
            a (Optional[float]): The a parameter.
            b (Optional[float]): The b parameter.
            k_max (Optional[int]): Number of elements of the series to compute.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)
        self.a = a
        self.b = b
        self.k_max = k_max

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i=1}^D \left( \sum_{k=0}^{k_{max}} a^k \cos\left( 2 \pi b^k ( x_i + 0.5) \right) \right) - D \sum_{k=0}^{k_{max}} a^k \cos \left( 2 \pi b^k \cdot 0.5 \right)$'''

    def _evaluate(self, x):
        val1 = 0.0
        for i in range(self.dimension):
            val = 0.0
            for k in range(self.k_max):
                val += self.a ** k * np.cos(2.0 * np.pi * self.b ** k * (x[i] + 0.5))
            val1 += val
        val2 = 0.0
        for k in range(self.k_max):
            val2 += self.a ** k * np.cos(2 * np.pi * self.b ** k * 0.5)
        return val1 - self.dimension * val2



# encoding=utf8
import logging

from niapy.algorithms.algorithm import Algorithm
from niapy.util import levy_flight

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['FlowerPollinationAlgorithm']


class FlowerPollinationAlgorithm(Algorithm):
    r"""Implementation of Flower Pollination algorithm.

    Algorithm:
        Flower Pollination algorithm

    Date:
        2018

    Authors:
        Dusan Fister, Iztok Fister Jr. and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Yang, Xin-She. "Flower pollination algorithm for global optimization. International conference on unconventional computing and natural computation. Springer, Berlin, Heidelberg, 2012.

    References URL:
        Implementation is based on the following MATLAB code: https://www.mathworks.com/matlabcentral/fileexchange/45112-flower-pollination-algorithm?requestedDomain=true

    Attributes:
        Name (List[str]): List of strings representing algorithm names.
        p (float): Switch probability.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['FlowerPollinationAlgorithm', 'FPA']

    @staticmethod
    def info():
        r"""Get default information of algorithm.

        Returns:
            str: Basic information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Yang, Xin-She. "Flower pollination algorithm for global optimization. International conference on unconventional computing and natural computation. Springer, Berlin, Heidelberg, 2012."""

    def __init__(self, population_size=20, p=0.8, *args, **kwargs):
        """Initialize FlowerPollinationAlgorithm.

        Args:
            population_size (int): Population size.
            p (float): Switch probability.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.p = p

    def set_parameters(self, population_size=25, p=0.8, **kwargs):
        r"""Set core parameters of FlowerPollinationAlgorithm algorithm.

        Args:
            population_size (int): Population size.
            p (float): Switch probability.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.p = p

    def init_population(self, task):
        """Initialize population."""
        pop, fpop, d = super().init_population(task)
        d.update({'solutions': pop.copy()})
        return pop, fpop, d

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of FlowerPollinationAlgorithm algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population fitness/function values.
            best_x (numpy.ndarray): Global best solution.
            best_fitness (float): Global best solution function/fitness value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New populations fitness/function values.
                3. New global best solution.
                4. New global best solution fitness/objective value.
                5. Additional arguments.

        """
        solutions = params.pop('solutions')

        for i in range(self.population_size):
            if self.random() > self.p:
                step_size = levy_flight(size=task.dimension, rng=self.rng)
                solutions[i] = population[i] + step_size * (population[i] - best_x)
                solutions[i] = task.repair(solutions[i], rng=self.rng)
            else:
                j, k = self.rng.choice(self.population_size, size=2, replace=False)
                solutions[i] += self.random() * (population[j] - population[k])
                solutions[i] = task.repair(solutions[i], rng=self.rng)
            f_i = task.eval(solutions[i])
            if f_i <= population_fitness[i]:
                population[i], population_fitness[i] = solutions[i].copy(), f_i
            if f_i <= best_fitness:
                best_x, best_fitness = solutions[i].copy(), f_i
        return population, population_fitness, best_x, best_fitness, {'solutions': solutions}



# encoding=utf8

"""Implementations of High Conditioned Elliptic functions."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Elliptic']


class Elliptic(Problem):
    r"""Implementations of High Conditioned Elliptic functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **High Conditioned Elliptic Function**

        :math:`f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    def __init__(self, dimension=4, lower=-100.0, upper=100.0, *args, **kwargs):
        r"""Initialize High Conditioned Elliptic problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2$'''

    def _evaluate(self, x):
        indices = np.arange(self.dimension)
        return np.sum(1000000.0 ** (indices / (self.dimension - 1)) * x)



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm, Individual, default_individual_init

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['GeneticAlgorithm', 'tournament_selection', 'roulette_selection', 'two_point_crossover',
           'multi_point_crossover',
           'uniform_crossover', 'uniform_mutation', 'creep_mutation', 'crossover_uros', 'mutation_uros']


def tournament_selection(pop, _ic, ts, _x_b, rng):
    r"""Tournament selection method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        _ic (int): Index of current individual in population. (Unused)
        ts (int): Tournament size.
        _x_b (numpy.ndarray): Global best individual. (Unused)
        rng (numpy.random.Generator): Random generator.

    Returns:
        Individual: Winner of the tournament.

    """
    comps = [pop[i] for i in rng.choice(len(pop), ts, replace=False)]
    return comps[np.argmin([c.f for c in comps])]


def roulette_selection(pop, ic, _ts, x_b, rng):
    r"""Roulette selection method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual in population.
        _ts (int): Unused argument.
        x_b (numpy.ndarray): Global best individual.
        rng (numpy.random.Generator): Random generator.

    Returns:
        Individual: selected individual.

    """
    f = np.sum([x.f for x in pop])
    qi = np.sum([pop[i].f / f for i in range(ic + 1)])
    return pop[ic].x if rng.random() < qi else x_b


def two_point_crossover(pop, ic, _cr, rng):
    r"""Two point crossover method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual.
        _cr (float): Crossover probability. (Unused)
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    io = ic
    while io != ic:
        io = rng.integers(len(pop))
    r = np.sort(rng.choice(len(pop[ic]), 2))
    x = pop[ic].x
    x[r[0]:r[1]] = pop[io].x[r[0]:r[1]]
    return np.asarray(x)


def multi_point_crossover(pop, ic, n, rng):
    r"""Multi point crossover method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual.
        n (flat): Number of points.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    io = ic
    while io != ic:
        io = rng.integers(len(pop))
    r, x = np.sort(rng.choice(len(pop[ic]), 2 * n)), pop[ic].x
    for i in range(n):
        x[r[2 * i]:r[2 * i + 1]] = pop[io].x[r[2 * i]:r[2 * i + 1]]
    return np.asarray(x)


def uniform_crossover(pop, ic, cr, rng):
    r"""Uniform crossover method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual.
        cr (float): Crossover probability.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    io = ic
    while io != ic:
        io = rng.integers(len(pop))
    j = rng.integers(len(pop[ic]))
    x = [pop[io][i] if rng.random() < cr or i == j else pop[ic][i] for i in range(len(pop[ic]))]
    return np.asarray(x)


def crossover_uros(pop, ic, cr, rng):
    r"""Crossover made by Uros Mlakar.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual.
        cr (float): Crossover probability.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    io = ic
    while io != ic:
        io = rng.integers(len(pop))
    alpha = cr + (1 + 2 * cr) * rng.random(len(pop[ic]))
    x = alpha * pop[ic] + (1 - alpha) * pop[io]
    return x


def uniform_mutation(pop, ic, mr, task, rng):
    r"""Uniform mutation method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of current individual.
        mr (float): Mutation probability.
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    j = rng.integers(task.dimension)
    nx = [rng.uniform(task.lower[i], task.upper[i]) if rng.random() < mr or i == j else pop[ic][i] for i in
          range(task.dimension)]
    return np.asarray(nx)


def mutation_uros(pop, ic, mr, task, rng):
    r"""Mutation method made by Uros Mlakar.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        ic (int): Index of individual.
        mr (float): Mutation rate.
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    return np.fmin(np.fmax(rng.normal(pop[ic], mr * task.range), task.lower), task.upper)


def creep_mutation(pop, _ic, mr, task, rng):
    r"""Creep mutation method.

    Args:
        pop (numpy.ndarray[Individual]): Current population.
        _ic (int): Index of current individual. (Unused)
        mr (float): Mutation probability.
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: New genotype.

    """
    ic, j = rng.integers(len(pop)), rng.integers(task.dimension)
    nx = [rng.uniform(task.lower[i], task.upper[i]) if rng.random() < mr or i == j else pop[ic][i] for i in
          range(task.dimension)]
    return np.asarray(nx)


class GeneticAlgorithm(Algorithm):
    r"""Implementation of Genetic Algorithm.

    Algorithm:
        Genetic algorithm

    Date:
        2018

    Author:
        Klemen Berkovič

    Reference paper:
        Goldberg, David (1989). Genetic Algorithms in Search, Optimization and Machine Learning. Reading, MA: Addison-Wesley Professional.

    License:
        MIT

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        tournament_size (int): Tournament size.
        mutation_rate (float): Mutation rate.
        crossover_rate (float): Crossover rate.
        selection (Callable[[numpy.ndarray[Individual], int, int, Individual, numpy.random.Generator], Individual]): selection operator.
        crossover (Callable[[numpy.ndarray[Individual], int, float, numpy.random.Generator], Individual]): Crossover operator.
        mutation (Callable[[numpy.ndarray[Individual], int, float, Task, numpy.random.Generator], Individual]): Mutation operator.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['GeneticAlgorithm', 'GA']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information of algorithm.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""On info"""

    def __init__(self, population_size=25, tournament_size=5, mutation_rate=0.25, crossover_rate=0.25,
                 selection=tournament_selection, crossover=uniform_crossover, mutation=uniform_mutation, *args, **kwargs):
        """Initialize GeneticAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            tournament_size (Optional[int]): Tournament selection.
            mutation_rate (Optional[int]): Mutation rate.
            crossover_rate (Optional[float]): Crossover rate.
            selection (Optional[Callable[[numpy.ndarray[Individual], int, int, Individual, numpy.random.Generator], Individual]]): Selection operator.
            crossover (Optional[Callable[[numpy.ndarray[Individual], int, float, numpy.random.Generator], Individual]]): Crossover operator.
            mutation (Optional[Callable[[numpy.ndarray[Individual], int, float, Task, numpy.random.Generator], Individual]]): Mutation operator.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`
            * selection:
                * :func:`niapy.algorithms.basic.tournament_selection`
                * :func:`niapy.algorithms.basic.roulette_selection`
            * Crossover:
                * :func:`niapy.algorithms.basic.uniform_crossover`
                * :func:`niapy.algorithms.basic.two_point_crossover`
                * :func:`niapy.algorithms.basic.multi_point_crossover`
                * :func:`niapy.algorithms.basic.crossover_uros`
            * Mutations:
                * :func:`niapy.algorithms.basic.uniform_mutation`
                * :func:`niapy.algorithms.basic.creep_mutation`
                * :func:`niapy.algorithms.basic.mutation_uros`

        """
        super().__init__(population_size,
                         individual_type=kwargs.pop('individual_type', Individual),
                         initialization_function=kwargs.pop('initialization_function', default_individual_init),
                         *args, **kwargs)
        self.tournament_size = tournament_size
        self.mutation_rate = mutation_rate
        self.crossover_rate = crossover_rate
        self.selection = selection
        self.crossover = crossover
        self.mutation = mutation

    def set_parameters(self, population_size=25, tournament_size=5, mutation_rate=0.25, crossover_rate=0.25,
                       selection=tournament_selection, crossover=uniform_crossover, mutation=uniform_mutation, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            tournament_size (Optional[int]): Tournament selection.
            mutation_rate (Optional[int]): Mutation rate.
            crossover_rate (Optional[float]): Crossover rate.
            selection (Optional[Callable[[numpy.ndarray[Individual], int, int, Individual, numpy.random.Generator], Individual]]): selection operator.
            crossover (Optional[Callable[[numpy.ndarray[Individual], int, float, numpy.random.Generator], Individual]]): Crossover operator.
            mutation (Optional[Callable[[numpy.ndarray[Individual], int, float, Task, numpy.random.Generator], Individual]]): Mutation operator.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`
            * selection:
                * :func:`niapy.algorithms.basic.tournament_selection`
                * :func:`niapy.algorithms.basic.roulette_selection`
            * Crossover:
                * :func:`niapy.algorithms.basic.uniform_crossover`
                * :func:`niapy.algorithms.basic.two_point_crossover`
                * :func:`niapy.algorithms.basic.multi_point_crossover`
                * :func:`niapy.algorithms.basic.crossover_uros`
            * Mutations:
                * :func:`niapy.algorithms.basic.uniform_mutation`
                * :func:`niapy.algorithms.basic.creep_mutation`
                * :func:`niapy.algorithms.basic.mutation_uros`

        """
        super().set_parameters(population_size=population_size,
                               individual_type=kwargs.pop('individual_type', Individual),
                               initialization_function=kwargs.pop('initialization_function', default_individual_init),
                               **kwargs)
        self.tournament_size = tournament_size
        self.mutation_rate = mutation_rate
        self.crossover_rate = crossover_rate
        self.selection = selection
        self.crossover = crossover
        self.mutation = mutation

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of GeneticAlgorithm algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current populations fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individuals function/fitness value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New populations function/fitness values.
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Additional arguments.

        """
        new_pop = np.empty(self.population_size, dtype=object)
        for i in range(self.population_size):
            ind = self.individual_type(x=self.selection(population, i, self.tournament_size, best_x, self.rng), e=False)
            ind.x = self.crossover(population, i, self.crossover_rate, self.rng)
            ind.x = self.mutation(population, i, self.mutation_rate, task, self.rng)
            ind.evaluate(task, rng=self.rng)
            new_pop[i] = ind
            if new_pop[i].f < best_fitness:
                best_x, best_fitness = self.get_best(new_pop[i], new_pop[i].f, best_x, best_fitness)
        return new_pop, np.asarray([i.f for i in new_pop]), best_x, best_fitness, {}



# encoding=utf8
import logging

from niapy.algorithms.basic import BatAlgorithm
from niapy.algorithms.basic.de import cross_best1

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.modified')
logger.setLevel('INFO')

__all__ = ['HybridBatAlgorithm']


class HybridBatAlgorithm(BatAlgorithm):
    r"""Implementation of Hybrid bat algorithm.

    Algorithm:
        Hybrid bat algorithm

    Date:
        2018

    Author:
        Grega Vrbančič and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Fister Jr., Iztok and Fister, Dusan and Yang, Xin-She. "A Hybrid Bat Algorithm". Elektrotehniški vestnik, 2013. 1-7.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        F (float): Scaling factor.
        CR (float): Crossover.

    See Also:
        * :class:`niapy.algorithms.basic.BatAlgorithm`

    """

    Name = ['HybridBatAlgorithm', 'HBA']

    @staticmethod
    def info():
        r"""Get basic information about the algorithm.

        Returns:
            str: Basic information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Fister Jr., Iztok and Fister, Dusan and Yang, Xin-She. "A Hybrid Bat Algorithm". Elektrotehniški vestnik, 2013. 1-7."""

    def __init__(self, differential_weight=0.50, crossover_probability=0.90, strategy=cross_best1, *args, **kwargs):
        """Initialize HybridBatAlgorithm.

        Args:
            differential_weight (Optional[float]): Differential weight.
            crossover_probability (Optional[float]): Crossover rate.
            strategy (Optional[Callable]): DE Crossover and mutation strategy.

        See Also:
            * :func:`niapy.algorithms.basic.BatAlgorithm.set_parameters`

        """
        super().__init__(*args, **kwargs)
        self.differential_weight = differential_weight
        self.crossover_probability = crossover_probability
        self.strategy = strategy

    def set_parameters(self, differential_weight=0.50, crossover_probability=0.90, strategy=cross_best1, **kwargs):
        r"""Set core parameters of HybridBatAlgorithm algorithm.

        Args:
            differential_weight (Optional[float]): Differential weight.
            crossover_probability (Optional[float]): Crossover rate.
            strategy (Callable): DE Crossover and mutation strategy.

        See Also:
            * :func:`niapy.algorithms.basic.BatAlgorithm.set_parameters`

        """
        super().set_parameters(**kwargs)
        self.differential_weight = differential_weight
        self.crossover_probability = crossover_probability
        self.strategy = strategy

    def local_search(self, best, task, i=None, population=None, **kwargs):
        r"""Improve the best solution.

        Args:
            best (numpy.ndarray): Global best individual.
            task (Task): Optimization task.
            i (int): Index of current individual.
            population (numpy.ndarray): Current best population.

        Returns:
            numpy.ndarray: New solution based on global best individual.

        """
        return task.repair(self.strategy(population, i, self.differential_weight, self.crossover_probability, self.rng, best), rng=self.rng)



# encoding=utf8
import logging
import math

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['CatSwarmOptimization']


class CatSwarmOptimization(Algorithm):
    r"""Implementation of Cat swarm optimization algorithm.

    **Algorithm:** Cat swarm optimization

    **Date:** 2019

    **Author:** Mihael Baketarić

    **License:** MIT

    **Reference paper:** Chu, S. C., Tsai, P. W., & Pan, J. S. (2006). Cat swarm optimization.
    In Pacific Rim international conference on artificial intelligence (pp. 854-858). Springer, Berlin, Heidelberg.

    """

    Name = ['CatSwarmOptimization', 'CSO']

    @staticmethod
    def info():
        r"""Get algorithm information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Chu, S. C., Tsai, P. W., & Pan, J. S. (2006). Cat swarm optimization.
        In Pacific Rim international conference on artificial intelligence (pp. 854-858).
        Springer, Berlin, Heidelberg."""

    def __init__(self, population_size=30, mixture_ratio=0.1, c1=2.05, smp=3, spc=True, cdc=0.85, srd=0.2,
                 max_velocity=1.9, *args, **kwargs):
        """Initialize CatSwarmOptimization.

        Args:
            population_size (int): Number of individuals in population.
            mixture_ratio (float): Mixture ratio.
            c1 (float): Constant in tracing mode.
            smp (int): Seeking memory pool.
            spc (bool): Self-position considering.
            cdc (float): Decides how many dimensions will be varied.
            srd (float): Seeking range of the selected dimension.
            max_velocity (float): Maximal velocity.

            See Also:
                * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.mixture_ratio = mixture_ratio
        self.c1 = c1
        self.smp = smp
        self.spc = spc
        self.cdc = cdc
        self.srd = srd
        self.max_velocity = max_velocity

    def set_parameters(self, population_size=30, mixture_ratio=0.1, c1=2.05, smp=3, spc=True, cdc=0.85, srd=0.2,
                       max_velocity=1.9, **kwargs):
        r"""Set the algorithm parameters.

        Args:
            population_size (int): Number of individuals in population.
            mixture_ratio (float): Mixture ratio.
            c1 (float): Constant in tracing mode.
            smp (int): Seeking memory pool.
            spc (bool): Self-position considering.
            cdc (float): Decides how many dimensions will be varied.
            srd (float): Seeking range of the selected dimension.
            max_velocity (float): Maximal velocity.

            See Also:
                * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size, **kwargs)
        self.mixture_ratio = mixture_ratio
        self.c1 = c1
        self.smp = smp
        self.spc = spc
        self.cdc = cdc
        self.srd = srd
        self.max_velocity = max_velocity

    def init_population(self, task):
        r"""Initialize population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. Initialized population.
                2. Initialized populations fitness/function values.
                3. Additional arguments:
                    * Dictionary of modes (seek or trace) and velocities for each cat
        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        pop, fpop, d = super().init_population(task)
        d['velocities'] = self.uniform(-self.max_velocity, self.max_velocity, (self.population_size, task.dimension))
        return pop, fpop, d

    def random_seek_trace(self):
        r"""Set cats into seeking/tracing mode randomly.

        Returns:
            numpy.ndarray: One or zero. One means tracing mode. Zero means seeking mode. Length of list is equal to population_size.

        """
        modes = np.zeros(self.population_size, dtype=np.int32)
        indices = self.rng.choice(self.population_size, int(self.population_size * self.mixture_ratio), replace=False)
        modes[indices] = 1
        return modes

    def weighted_selection(self, weights):
        r"""Random selection considering the weights.

        Args:
            weights (numpy.ndarray): weight for each potential position.

        Returns:
            int: index of selected next position.

        """
        cumulative_sum = np.cumsum(weights)
        return np.argmax(cumulative_sum >= (self.random() * cumulative_sum[-1]))

    def seeking_mode(self, task, cat, cat_fitness, pop, fpop, fxb):
        r"""Seeking mode.

        Args:
            task (Task): Optimization task.
            cat (numpy.ndarray): Individual from population.
            cat_fitness (float): Current individual's fitness/function value.
            pop (numpy.ndarray): Current population.
            fpop (numpy.ndarray): Current population fitness/function values.
            fxb (float): Current best cat fitness/function value.

        Returns:
            Tuple[numpy.ndarray, float, numpy.ndarray, float]:
                1. Updated individual's position
                2. Updated individual's fitness/function value
                3. Updated global best position
                4. Updated global best fitness/function value

        """
        cat_copies = []
        cat_copies_fs = []
        for j in range(self.smp - 1 if self.spc else self.smp):
            cat_copies.append(cat.copy())
            indexes = np.arange(task.dimension)
            self.rng.shuffle(indexes)
            to_vary_indexes = indexes[:int(task.dimension * self.cdc)]
            if self.integers(2) == 1:
                cat_copies[j][to_vary_indexes] += cat_copies[j][to_vary_indexes] * self.srd
            else:
                cat_copies[j][to_vary_indexes] -= cat_copies[j][to_vary_indexes] * self.srd
            cat_copies[j] = task.repair(cat_copies[j])
            cat_copies_fs.append(task.eval(cat_copies[j]))
        if self.spc:
            cat_copies.append(cat.copy())
            cat_copies_fs.append(cat_fitness)

        cat_copies_select_probs = np.ones(len(cat_copies))
        worst_fitness = np.max(cat_copies_fs)
        best_fitness = np.min(cat_copies_fs)
        if any(x != cat_copies_fs[0] for x in cat_copies_fs):
            fb = worst_fitness
            if math.isinf(fb):
                cat_copies_select_probs = np.full(len(cat_copies), fb)
            else:
                cat_copies_select_probs = np.abs(cat_copies_fs - fb) / (worst_fitness - best_fitness)
        if best_fitness < fxb:
            ind = self.integers(self.population_size)
            pop[ind] = cat_copies[np.where(cat_copies_fs == best_fitness)[0][0]]
            fpop[ind] = best_fitness
        sel_index = self.weighted_selection(cat_copies_select_probs)
        return cat_copies[sel_index], cat_copies_fs[sel_index], pop, fpop

    def tracing_mode(self, task, cat, velocity, xb):
        r"""Tracing mode.

        Args:
            task (Task): Optimization task.
            cat (numpy.ndarray): Individual from population.
            velocity (numpy.ndarray): Velocity of individual.
            xb (numpy.ndarray): Current best individual.
        Returns:
            Tuple[numpy.ndarray, float, numpy.ndarray]:
                1. Updated individual's position
                2. Updated individual's fitness/function value
                3. Updated individual's velocity vector

        """
        new_velocity = np.clip(velocity + (self.random(len(velocity)) * self.c1 * (xb - cat)),
                               -self.max_velocity, self.max_velocity)
        cat_new = task.repair(cat + new_velocity)
        return cat_new, task.eval(cat_new), new_velocity

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Cat Swarm Optimization algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population fitness/function values.
            best_x (numpy.ndarray): Current best individual.
            best_fitness (float): Current best cat fitness/function value.
            **params (Dict[str, Any]): Additional function arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * velocities (numpy.ndarray): velocities of cats.

        """
        modes = self.random_seek_trace()
        velocities = params.pop('velocities')

        pop_copies = population.copy()
        for k in range(len(pop_copies)):
            if modes[k] == 0:
                pop_copies[k], population_fitness[k], pop_copies, population_fitness = self.seeking_mode(task,
                                                                                                         pop_copies[k],
                                                                                                         population_fitness[k],
                                                                                                         pop_copies,
                                                                                                         population_fitness,
                                                                                                         best_fitness)
            else:  # if cat in tracing mode
                pop_copies[k], population_fitness[k], velocities[k] = self.tracing_mode(task,
                                                                                        pop_copies[k],
                                                                                        velocities[k],
                                                                                        best_x)
        best_index = np.argmin(population_fitness)
        if population_fitness[best_index] < best_fitness:
            best_x, best_fitness = pop_copies[best_index].copy(), population_fitness[best_index]
        return pop_copies, population_fitness, best_x, best_fitness, {'velocities': velocities}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm
from niapy.util import euclidean

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['GlowwormSwarmOptimization', 'GlowwormSwarmOptimizationV1', 'GlowwormSwarmOptimizationV2',
           'GlowwormSwarmOptimizationV3']


class GlowwormSwarmOptimization(Algorithm):
    r"""Implementation of glowworm swarm optimization.

    Algorithm:
        Glowworm Swarm Optimization Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        https://www.springer.com/gp/book/9783319515946

    Reference paper:
        Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        l0 (float): Initial luciferin quantity for each glowworm.
        nt (float): Number of neighbors.
        rho (float): Luciferin decay constant.
        gamma (float): Luciferin enhancement constant.
        beta (float): Constant.
        s (float): Step size.
        distance (Callable[[numpy.ndarray, numpy.ndarray], float]]): Measure distance between two individuals.

    See Also:
        * :class:`NiaPy.algorithms.algorithm.Algorithm`

    """

    Name = ['GlowwormSwarmOptimization', 'GSO']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information.

        """
        return r"""Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017."""

    def __init__(self, population_size=25, l0=5, nt=5, rho=0.4, gamma=0.6, beta=0.08, s=0.03, distance=euclidean, *args,
                 **kwargs):
        """Initialize GlowwormSwarmOptimization.

        Args:
            population_size (Optional[int]): Number of glowworms in population.
            l0 (Optional[float]): Initial luciferin quantity for each glowworm.
            nt (Optional[int]): Number of neighbors.
            rho (Optional[float]): Luciferin decay constant.
            gamma (Optional[float]): Luciferin enhancement constant.
            beta (Optional[float]): Constant.
            s (Optional[float]): Step size.
            distance (Optional[Callable[[numpy.ndarray, numpy.ndarray], float]]]): Measure distance between two individuals.

        """
        super().__init__(population_size, *args, **kwargs)
        self.l0 = l0
        self.nt = nt
        self.rho = rho
        self.gamma = gamma
        self.beta = beta
        self.s = s
        self.distance = distance

    def set_parameters(self, population_size=25, l0=5, nt=5, rho=0.4, gamma=0.6, beta=0.08, s=0.03, distance=euclidean,
                       **kwargs):
        r"""Set the arguments of an algorithm.

        Args:
            population_size (Optional[int]): Number of glowworms in population.
            l0 (Optional[float]): Initial luciferin quantity for each glowworm.
            nt (Optional[int]): Number of neighbors.
            rho (Optional[float]): Luciferin decay constant.
            gamma (Optional[float]): Luciferin enhancement constant.
            beta (Optional[float]): Constant.
            s (Optional[float]): Step size.
            distance (Optional[Callable[[numpy.ndarray, numpy.ndarray], float]]]): Measure distance between two individuals.

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.l0 = l0
        self.nt = nt
        self.rho = rho
        self.gamma = gamma
        self.beta = beta
        self.s = s
        self.distance = distance

    def get_parameters(self):
        r"""Get algorithms parameters values.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = super().get_parameters()
        d.update({
            'l0': self.l0,
            'nt': self.nt,
            'rho': self.rho,
            'gamma': self.gamma,
            'beta': self.beta,
            's': self.s,
            'distance': self.distance
        })
        return d

    def get_neighbors(self, i, r, glowworms, luciferin):
        r"""Get neighbours of glowworm.

        Args:
            i (int): Index of glowworm.
            r (float): Neighborhood distance.
            glowworms (numpy.ndarray):
            luciferin (numpy.ndarray[float]): Luciferin value of glowworm.

        Returns:
            numpy.ndarray[int]: Indexes of neighborhood glowworms.

        """
        neighbors = np.zeros(self.population_size, dtype=np.int32)
        for j, gw in enumerate(glowworms):
            neighbors[j] = 1 if i != j and self.distance(glowworms[i], gw) <= r and luciferin[i] >= luciferin[j] else 0
        return neighbors

    def probabilities(self, i, neighbors, luciferin):
        r"""Calculate probabilities for glowworm to movement.

        Args:
            i (int): Index of glowworm to search for probable movement.
            neighbors (numpy.ndarray[float]):
            luciferin (numpy.ndarray[float]):

        Returns:
            numpy.ndarray[float]: Probabilities for each glowworm in swarm.

        """
        d = np.sum(luciferin[neighbors == 1] - luciferin[i]) + np.finfo(float).eps
        probabilities = np.zeros(self.population_size)
        probabilities[neighbors == 1] = (luciferin[neighbors == 1] - luciferin[i]) / d
        return probabilities

    def move_select(self, pb, i):
        r"""Get move index for the i-th glowworm.

        Args:
            pb (numpy.ndarray): Probabilities.
            i (int): Index of the glowworm.

        Returns:
            int: Index i-th glowworm will move towards.

        """
        r, b_l, b_u = self.random(), 0, 0
        for j in range(self.population_size):
            b_l, b_u = b_u, b_u + pb[i]
            if b_l < r < b_u:
                return j
        return self.integers(self.population_size)

    def calculate_luciferin(self, luciferin, fitness):
        return (1 - self.rho) * luciferin + self.gamma * fitness

    def range_update(self, range_, neighbors, sensing_range):
        """Update range."""
        return range_ + self.beta * (self.nt - np.sum(neighbors))

    def init_population(self, task):
        r"""Initialize population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. Initialized population of glowworms.
                2. Initialized populations function/fitness values.
                3. Additional arguments:
                    * luciferin (numpy.ndarray): Luciferin values of glowworms.
                    * ranges (numpy.ndarray): Ranges.
                    * sensing_range (float): Sensing range.

        """
        population, fitness, d = super().init_population(task)
        sensing_range = euclidean(np.zeros(task.dimension), task.range)
        luciferin = np.full(self.population_size, self.l0)
        ranges = np.full(self.population_size, sensing_range)
        d.update({'luciferin': luciferin, 'ranges': ranges, 'sensing_range': sensing_range})
        return population, fitness, d

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of GlowwormSwarmOptimization algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current populations fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individuals function/fitness value.
            **params Dict[str, Any]: Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. Initialized population of glowworms.
                2. Initialized populations function/fitness values.
                3. New global best solution
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * luciferin (numpy.ndarray): Luciferin values of glowworms.
                    * ranges (numpy.ndarray): Ranges.
                    * sensing_range (float): Sensing range.

        """
        luciferin = params.pop('luciferin')
        ranges = params.pop('ranges')
        sensing_range = params.pop('sensing_range')

        old_population, old_ranges = np.copy(population), np.copy(ranges)
        luciferin = self.calculate_luciferin(luciferin, population_fitness)
        neighbors = [self.get_neighbors(i, old_ranges[i], old_population, luciferin) for i in range(self.population_size)]
        probabilities = [self.probabilities(i, neighbors[i], luciferin) for i in range(self.population_size)]
        j = [self.move_select(probabilities[i], i) for i in range(self.population_size)]
        for i in range(self.population_size):
            distance = self.distance(old_population[j[i]], old_population[i])
            new_glowworm = old_population[i] + self.s * ((old_population[j[i]] - old_population[i]) / (distance + 1e-31))
            population[i] = task.repair(new_glowworm, rng=self.rng)
        for i in range(self.population_size):
            ranges[i] = max(0.0, min(sensing_range, self.range_update(old_ranges[i], neighbors[i], sensing_range)))
        population_fitness = np.apply_along_axis(task.eval, 1, population)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {'luciferin': luciferin,
                                                                      'ranges': ranges,
                                                                      'sensing_range': sensing_range}


class GlowwormSwarmOptimizationV1(GlowwormSwarmOptimization):
    r"""Implementation of glowworm swarm optimization.

    Algorithm:
        Glowworm Swarm Optimization Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        https://www.springer.com/gp/book/9783319515946

    Reference paper:
        Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017.

    Attributes:
        Name (List[str]): List of strings representing algorithm names.

    See Also:
        * :class:`NiaPy.algorithms.basic.GlowwormSwarmOptimization`

    """

    Name = ['GlowwormSwarmOptimizationV1', 'GSOv1']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information.

        """
        return r"""Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017."""

    def calculate_luciferin(self, luciferin, fitness):
        r = super().calculate_luciferin(luciferin, fitness)
        return np.fmax(0.0, r)

    def range_update(self, range_, neighbors, sensing_range):
        """Update range."""
        return sensing_range / (1 + self.beta * (np.sum(neighbors) / (np.pi * sensing_range ** 2)))


class GlowwormSwarmOptimizationV2(GlowwormSwarmOptimization):
    r"""Implementation of glowworm swarm optimization.

    Algorithm:
        Glowworm Swarm Optimization Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        https://www.springer.com/gp/book/9783319515946

    Reference paper:
        Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017.

    Attributes:
        Name (List[str]): List of strings representing algorithm names.
        alpha (float): --

    See Also:
        * :class:`NiaPy.algorithms.basic.GlowwormSwarmOptimization`

    """

    Name = ['GlowwormSwarmOptimizationV2', 'GSOv2']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information.

        """
        return r"""Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017."""

    def __init__(self, alpha=0.2, *args, **kwargs):
        """Initialize GlowwormSwarmOptimizationV2.

        Args:
            alpha (Optional[float]): Alpha parameter.

        """
        super().__init__(*args, **kwargs)
        self.alpha = alpha

    def set_parameters(self, alpha=0.2, **kwargs):
        r"""Set core parameters for GlowwormSwarmOptimizationV2 algorithm.

        Args:
            alpha (Optional[float]): Alpha parameter.

        See Also:
            * :func:`NiaPy.algorithms.basic.GlowwormSwarmOptimization.set_parameters`

        """
        super().set_parameters(**kwargs)
        self.alpha = alpha

    def range_update(self, range_, neighbors, sensing_range):
        """Update range."""
        return self.alpha + (sensing_range - self.alpha) / (1 + self.beta * np.sum(neighbors))


class GlowwormSwarmOptimizationV3(GlowwormSwarmOptimization):
    r"""Implementation of glowworm swarm optimization.

    Algorithm:
        Glowworm Swarm Optimization Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        https://www.springer.com/gp/book/9783319515946

    Reference paper:
        Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017.

    Attributes:
        Name (List[str]): List of strings representing algorithm names.
        beta1 (float): --

    See Also:
        * :class:`NiaPy.algorithms.basic.GlowwormSwarmOptimization`

    """

    Name = ['GlowwormSwarmOptimizationV3', 'GSOv3']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information.

        """
        return r"""Kaipa, Krishnanand N., and Debasish Ghose. Glowworm swarm optimization: theory, algorithms, and applications. Vol. 698. Springer, 2017."""

    def __init__(self, beta1=0.2, *args, **kwargs):
        """Initialize GlowwormSwarmOptimizationV3.

        Args:
            beta1 (Optional[float]): Beta1 parameter.

        """
        super().__init__(*args, **kwargs)
        self.beta1 = beta1

    def set_parameters(self, beta1=0.2, **kwargs):
        r"""Set core parameters for GlowwormSwarmOptimizationV3 algorithm.

        Args:
            beta1 (Optional[float]): Beta1 parameter.

        See Also:
            * :func:`NiaPy.algorithms.basic.GlowwormSwarmOptimization.set_parameters`

        """
        super().set_parameters(**kwargs)
        self.beta1 = beta1

    def range_update(self, range_, neighbors, sensing_range):
        """Update range."""
        return range_ + (self.beta * np.sum(neighbors)) if np.sum(neighbors) < self.nt else (-self.beta1 * np.sum(neighbors))



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.other')
logger.setLevel('INFO')

__all__ = ['SimulatedAnnealing', 'cool_delta', 'cool_linear']


def cool_delta(current_temperature, delta_temperature, **_kwargs):
    r"""Calculate new temperature by differences.

    Args:
        current_temperature (float):
        delta_temperature (float):

    Returns:
        float: New temperature.

    """
    return current_temperature - delta_temperature


def cool_linear(current_temperature, starting_temperature, max_evals, **_kwargs):
    r"""Calculate temperature with linear function.

    Args:
        current_temperature (float): Current temperature.
        starting_temperature (float):
        max_evals (int): Number of evaluations done.
        _kwargs (Dict[str, Any]): Additional arguments.

    Returns:
        float: New temperature.

    """
    return current_temperature - starting_temperature / max_evals


class SimulatedAnnealing(Algorithm):
    r"""Implementation of Simulated Annealing Algorithm.

    Algorithm:
        Simulated Annealing Algorithm

    Date:
        2018

    Authors:
        Jan Popič and Klemen Berkovič

    License:
        MIT

    Reference URL:

    Reference paper:

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        delta (float): Movement for neighbour search.
        starting_temperature (float); Starting temperature.
        delta_temperature (float): Change in temperature.
        cooling_method (Callable): Neighbourhood function.
        epsilon (float): Error value.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['SimulatedAnnealing', 'SA']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information of algorithm.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""None"""

    def __init__(self, delta=0.5, starting_temperature=2000, delta_temperature=0.8, cooling_method=cool_delta,
                 epsilon=1e-23, *args, **kwargs):
        """Initialize SimulatedAnnealing.

        Args:
            delta (Optional[float]): Movement for neighbour search.
            starting_temperature (Optional[float]); Starting temperature.
            delta_temperature (Optional[float]): Change in temperature.
            cooling_method (Optional[Callable]): Neighbourhood function.
            epsilon (Optional[float]): Error value.

        See Also
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        kwargs.pop('population_size', None)
        super().__init__(1, *args, **kwargs)
        self.delta = delta
        self.starting_temperature = starting_temperature
        self.delta_temperature = delta_temperature
        self.cooling_method = cooling_method
        self.epsilon = epsilon

    def set_parameters(self, delta=0.5, starting_temperature=2000, delta_temperature=0.8, cooling_method=cool_delta,
                       epsilon=1e-23, **kwargs):
        r"""Set the algorithm parameters/arguments.

        Args:
            delta (Optional[float]): Movement for neighbour search.
            starting_temperature (Optional[float]); Starting temperature.
            delta_temperature (Optional[float]): Change in temperature.
            cooling_method (Optional[Callable]): Neighbourhood function.
            epsilon (Optional[float]): Error value.

        See Also
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        kwargs.pop('population_size', None)
        super().set_parameters(population_size=1, **kwargs)
        self.delta = delta
        self.starting_temperature = starting_temperature
        self.delta_temperature = delta_temperature
        self.cooling_method = cooling_method
        self.epsilon = epsilon

    def get_parameters(self):
        r"""Get algorithms parameters values.

        Returns:
            Dict[str, Any]:

        See Also
            * :func:`niapy.algorithms.Algorithm.get_parameters`

        """
        d = Algorithm.get_parameters(self)
        d.update({
            'delta': self.delta,
            'delta_temperature': self.delta_temperature,
            'starting_temperature': self.starting_temperature,
            'epsilon': self.epsilon
        })
        return d

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, float, dict]:
            1. Initial solution
            2. Initial solutions fitness/objective value
            3. Additional arguments

        """
        x = task.lower + task.range * self.random(task.dimension)
        current_temperature, x_fit = self.starting_temperature, task.eval(x)
        return x, x_fit, {'current_temperature': current_temperature}

    def run_iteration(self, task, x, x_fit, best_x, best_fitness, **params):
        r"""Core function of the algorithm.

        Args:
            task (Task):
            x (numpy.ndarray):
            x_fit (float):
            best_x (numpy.ndarray):
            best_fitness (float):
            **params (dict): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, float, numpy.ndarray, float, dict]:
            1. New solution
            2. New solutions fitness/objective value
            3. New global best solution
            4. New global best solutions fitness/objective value
            5. Additional arguments

        """
        current_temperature = params.pop('current_temperature')
        c = task.repair(x - self.delta / 2 + self.random(task.dimension) * self.delta, rng=self.rng)
        c_fit = task.eval(c)
        delta_fit, r = c_fit - x_fit, self.random()
        if delta_fit < 0 or r < np.exp(delta_fit / current_temperature):
            x, x_fit = c, c_fit
        current_temperature = self.cooling_method(current_temperature, starting_temperature=self.starting_temperature,
                                                  delta_temperature=self.delta_temperature, max_evals=task.max_evals)
        best_x, best_fitness = self.get_best(x, x_fit, best_x, best_fitness)
        return x, x_fit, best_x, best_fitness, {'current_temperature': current_temperature}



# encoding=utf8
"""Implementations of Zakharov function."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Zakharov']


class Zakharov(Problem):
    r"""Implementations of Zakharov functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Zakharov Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-5, 10]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (0, \cdots, 0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4 \end{equation}

        Domain:
            $-5 \leq x_i \leq 10$

    Reference:
        https://www.sfu.ca/~ssurjano/zakharov.html

    """

    def __init__(self, dimension=4, lower=-5.0, upper=10.0, *args, **kwargs):
        r"""Initialize Zakharov problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4$'''

    def _evaluate(self, x):
        sum1 = np.sum(x * x)
        sum2 = np.sum(0.5 * np.arange(1, self.dimension + 1) * x)
        return sum1 + sum2 ** 2 + sum2 ** 4



# encoding=utf8
import logging

from niapy.algorithms.algorithm import Individual
from niapy.algorithms.basic.de import DifferentialEvolution, cross_best1, cross_rand1, cross_curr2best1, cross_best2, \
    cross_curr2rand1, multi_mutations
from niapy.util import objects_to_array

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.modified')
logger.setLevel('INFO')

__all__ = [
    'SolutionJDE',
    'SelfAdaptiveDifferentialEvolution',
    'MultiStrategySelfAdaptiveDifferentialEvolution',
]


class SolutionJDE(Individual):
    r"""Individual for jDE algorithm.

    Attributes:
        differential_weight (float): Scale factor.
        crossover_probability (float): Crossover probability.

    See Also:
        :class:`niapy.algorithms.Individual`

    """

    def __init__(self, differential_weight=2, crossover_probability=0.5, **kwargs):
        r"""Initialize SolutionJDE.

        Attributes:
            differential_weight (float): Scale factor.
            crossover_probability (float): Crossover probability.

        See Also:
            :func:`niapy.algorithm.Individual.__init__`

        """
        super().__init__(**kwargs)
        self.differential_weight = differential_weight
        self.crossover_probability = crossover_probability


class SelfAdaptiveDifferentialEvolution(DifferentialEvolution):
    r"""Implementation of Self-adaptive differential evolution algorithm.

    Algorithm:
        Self-adaptive differential evolution algorithm

    Date:
        2018

    Author:
        Uros Mlakar and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Brest, J., Greiner, S., Boskovic, B., Mernik, M., Zumer, V. Self-adapting control parameters in differential evolution: A comparative study on numerical benchmark problems. IEEE transactions on evolutionary computation, 10(6), 646-657, 2006.

    Attributes:
        Name (List[str]): List of strings representing algorithm name
        f_lower (float): Scaling factor lower limit.
        f_upper (float): Scaling factor upper limit.
        tao1 (float): Change rate for differential_weight parameter update.
        tao2 (float): Change rate for crossover_probability parameter update.

    See Also:
        * :class:`niapy.algorithms.basic.DifferentialEvolution`

    """

    Name = ['SelfAdaptiveDifferentialEvolution', 'jDE']

    @staticmethod
    def info():
        r"""Get algorithm information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Brest, J., Greiner, S., Boskovic, B., Mernik, M., Zumer, V. Self-adapting control parameters in differential evolution: A comparative study on numerical benchmark problems. IEEE transactions on evolutionary computation, 10(6), 646-657, 2006."""

    def __init__(self, f_lower=0.0, f_upper=1.0, tao1=0.4, tao2=0.2, *args, **kwargs):
        """Initialize SelfAdaptiveDifferentialEvolution.

        Args:
            f_lower (Optional[float]): Scaling factor lower limit.
            f_upper (Optional[float]): Scaling factor upper limit.
            tao1 (Optional[float]): Change rate for differential_weight parameter update.
            tao2 (Optional[float]): Change rate for crossover_probability parameter update.

        See Also:
            * :func:`niapy.algorithms.basic.DifferentialEvolution.__init__`

        """
        super().__init__(individual_type=kwargs.pop('individual_type', SolutionJDE), *args, **kwargs)
        self.f_lower = f_lower
        self.f_upper = f_upper
        self.tao1 = tao1
        self.tao2 = tao2

    def set_parameters(self, f_lower=0.0, f_upper=1.0, tao1=0.4, tao2=0.2, **kwargs):
        r"""Set the parameters of an algorithm.

        Args:
            f_lower (Optional[float]): Scaling factor lower limit.
            f_upper (Optional[float]): Scaling factor upper limit.
            tao1 (Optional[float]): Change rate for differential_weight parameter update.
            tao2 (Optional[float]): Change rate for crossover_probability parameter update.

        See Also:
            * :func:`niapy.algorithms.basic.DifferentialEvolution.set_parameters`

        """
        super().set_parameters(individual_type=kwargs.pop('individual_type', SolutionJDE), **kwargs)
        self.f_lower = f_lower
        self.f_upper = f_upper
        self.tao1 = tao1
        self.tao2 = tao2

    def get_parameters(self):
        r"""Get algorithm parameters.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = DifferentialEvolution.get_parameters(self)
        d.update({
            'f_lower': self.f_lower,
            'f_upper': self.f_upper,
            'tao1': self.tao1,
            'tao2': self.tao2
        })
        return d

    def adaptive_gen(self, x):
        r"""Adaptive update scale factor in crossover probability.

        Args:
            x (IndividualJDE): Individual to apply function on.

        Returns:
            Individual: New individual with new parameters

        """
        f = self.f_lower + self.random() * (self.f_upper - self.f_lower) if self.random() < self.tao1 else x.differential_weight
        cr = self.random() if self.random() < self.tao2 else x.crossover_probability
        return self.individual_type(x=x.x, differential_weight=f, crossover_probability=cr, e=False)

    def evolve(self, pop, xb, task, **_kwargs):
        r"""Evolve current population.

        Args:
            pop (numpy.ndarray[Individual]): Current population.
            xb (Individual): Global best individual.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: New population.

        """
        new_pop = objects_to_array([self.adaptive_gen(e) for e in pop])
        for i, e in enumerate(new_pop):
            new_pop[i].x = self.strategy(new_pop, i, e.differential_weight, e.crossover_probability, rng=self.rng, x_b=xb)
        for e in new_pop:
            e.evaluate(task, rng=self.random)
        return new_pop


class MultiStrategySelfAdaptiveDifferentialEvolution(SelfAdaptiveDifferentialEvolution):
    r"""Implementation of self-adaptive differential evolution algorithm with multiple mutation strategies.

    Algorithm:
        Self-adaptive differential evolution algorithm with multiple mutation strategies

    Date:
        2018

    Author:
        Klemen Berkovič

    License:
        MIT

    Attributes:
        Name (List[str]): List of strings representing algorithm name

    See Also:
        * :class:`niapy.algorithms.modified.SelfAdaptiveDifferentialEvolution`

    """

    Name = ['MultiStrategySelfAdaptiveDifferentialEvolution', 'MsjDE']

    def __init__(self, strategies=(cross_curr2rand1, cross_curr2best1, cross_rand1, cross_best1, cross_best2), *args,
                 **kwargs):
        """Initialize MultiStrategySelfAdaptiveDifferentialEvolution.

        Args:
            strategies (Optional[Iterable[Callable]]): Mutations strategies to use in algorithm.

        See Also:
            * :func:`niapy.algorithms.modified.SelfAdaptiveDifferentialEvolution.__init__`

        """
        super().__init__(strategy=kwargs.pop('strategy', multi_mutations), *args, **kwargs)
        self.strategies = strategies

    def set_parameters(self, strategies=(cross_curr2rand1, cross_curr2best1, cross_rand1, cross_best1, cross_best2),
                       **kwargs):
        r"""Set core parameters of MultiStrategySelfAdaptiveDifferentialEvolution algorithm.

        Args:
            strategies (Optional[Iterable[Callable]]): Mutations strategies to use in algorithm.

        See Also:
            * :func:`niapy.algorithms.modified.SelfAdaptiveDifferentialEvolution.set_parameters`

        """
        super().set_parameters(strategy=kwargs.pop('strategy', multi_mutations), **kwargs)
        self.strategies = strategies

    def evolve(self, pop, xb, task, **kwargs):
        r"""Evolve population with the help multiple mutation strategies.

        Args:
            pop (numpy.ndarray[Individual]): Current population.
            xb (Individual): Current best individual.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray[Individual]: New population of individuals.

        """
        return objects_to_array(
            [self.strategy(pop, i, xb, self.differential_weight, self.crossover_probability, self.rng, task, self.individual_type, self.strategies) for i in
             range(len(pop))])



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

__all__ = ['MonarchButterflyOptimization']

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')


class MonarchButterflyOptimization(Algorithm):
    r"""Implementation of Monarch Butterfly Optimization.

    Algorithm:
        Monarch Butterfly Optimization

    Date:
        2019

    Authors:
        Jan Banko

    License:
        MIT

    Reference paper:
        Wang, G. G., Deb, S., & Cui, Z. (2019). Monarch butterfly optimization. Neural computing and applications, 31(7), 1995-2014.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        PAR (float): Partition.
        PER (float): Period.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['MonarchButterflyOptimization', 'MBO']

    @staticmethod
    def info():
        r"""Get information of the algorithm.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.info`

        """
        return r"""Description: Monarch butterfly optimization algorithm is inspired by the migration behaviour of the monarch butterflies in nature.
        Authors: Wang, Gai-Ge & Deb, Suash & Cui, Zhihua.
        Year: 2015
        Main reference: Wang, G. G., Deb, S., & Cui, Z. (2019). Monarch butterfly optimization. Neural computing and applications, 31(7), 1995-2014."""

    def __init__(self, population_size=20, partition=5.0 / 12.0, period=1.2, *args, **kwargs):
        """Initialize MonarchButterflyOptimization.

        Args:
            population_size (Optional[int]): Population size.
            partition (Optional[int]): Partition.
            period (Optional[int]): Period.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.partition = partition
        self.period = period
        self.keep = 2
        self.bar = partition
        self.np1 = int(np.ceil(partition * population_size))
        self.np2 = population_size - self.np1

    def set_parameters(self, population_size=20, partition=5.0 / 12.0, period=1.2, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            partition (Optional[int]): Partition.
            period (Optional[int]): Period.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.partition = partition
        self.period = period
        self.keep = 2
        self.bar = partition
        self.np1 = int(np.ceil(partition * population_size))
        self.np2 = population_size - self.np1

    def get_parameters(self):
        r"""Get parameters values for the algorithm.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = Algorithm.get_parameters(self)
        d.update({
            'partition': self.partition,
            'period': self.period,
            'keep': self.keep,
            'bar': self.bar,
            'np1': self.np1,
            'np2': self.np2
        })
        return d

    def levy(self, _step_size, dimension):
        r"""Calculate levy flight.

        Args:
            _step_size (float): Size of the walk step.
            dimension (int): Number of dimensions.

        Returns:
            numpy.ndarray: Calculated values for levy flight.

        """
        delta_x = np.array([np.sum(np.tan(np.pi * self.uniform(0.0, 1.0, 10))) for _ in range(dimension)])
        return delta_x

    def migration_operator(self, dimension, np1, np2, butterflies):
        r"""Apply the migration operator.

        Args:
            dimension (int): Number of dimensions.
            np1 (int): Number of butterflies in Land 1.
            np2 (int): Number of butterflies in Land 2.
            butterflies (numpy.ndarray): Current butterfly population.

        Returns:
            numpy.ndarray: Adjusted butterfly population.

        """
        pop1 = np.copy(butterflies[:np1])
        pop2 = np.copy(butterflies[np1:])
        for k1 in range(np1):
            for i in range(dimension):
                r1 = self.random() * self.period
                if r1 <= self.partition:
                    r2 = self.integers(np1 - 1)
                    butterflies[k1, i] = pop1[r2, i]
                else:
                    r3 = self.integers(np2 - 1)
                    butterflies[k1, i] = pop2[r3, i]
        return butterflies

    def adjusting_operator(self, t, max_t, dimension, np1, np2, butterflies, best):
        r"""Apply the adjusting operator.

        Args:
            t (int): Current generation.
            max_t (int): Maximum generation.
            dimension (int): Number of dimensions.
            np1 (int): Number of butterflies in Land 1.
            np2 (int): Number of butterflies in Land 2.
            butterflies (numpy.ndarray): Current butterfly population.
            best (numpy.ndarray): The best butterfly currently.

        Returns:
            numpy.ndarray: Adjusted butterfly population.

        """
        pop2 = np.copy(butterflies[np1:])
        for k2 in range(np1, np1 + np2):
            scale = 1.0 / ((t + 1) ** 2)
            step_size = np.ceil(self.rng.exponential(2 * max_t))
            delta_x = self.levy(step_size, dimension)
            for i in range(dimension):
                if self.uniform(0.0, 1.0) >= self.partition:
                    butterflies[k2, i] = best[i]
                else:
                    r4 = self.integers(np2 - 1)
                    butterflies[k2, i] = pop2[r4, 1]
                    if self.uniform(0.0, 1.0) > self.bar:
                        butterflies[k2, i] += scale * (delta_x[i] - 0.5)
        return butterflies

    @staticmethod
    def evaluate_and_sort(task, butterflies):
        r"""Evaluate and sort the butterfly population.

        Args:
            task (Task): Optimization task
            butterflies (numpy.ndarray): Current butterfly population.

        Returns:
            numpy.ndarray: Tuple[numpy.ndarray, float, numpy.ndarray]:
                1. Best butterfly according to the evaluation.
                2. The best fitness value.
                3. Butterfly population.

        """
        fitness = np.apply_along_axis(task.eval, 1, butterflies)
        indices = np.argsort(fitness)
        butterflies = butterflies[indices]
        fitness = fitness[indices]

        return fitness, butterflies

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * current_best (numpy.ndarray): Current generation's best individual.

        See Also:
             * :func:`niapy.algorithms.Algorithm.init_population`

        """
        population, fitness, _ = super().init_population(task)
        sorted_indices = np.argsort(fitness)
        population = population[sorted_indices]
        fitness = fitness[sorted_indices]
        return population, fitness, {'current_best': population[0]}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Forest Optimization Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray[float]): Current population function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * current_best (numpy.ndarray): Current generation's best individual.

        """
        current_best = params.pop('current_best')

        elite = np.copy(population[:self.keep])
        max_t = task.max_iters if not np.isinf(task.max_iters) else task.max_evals / self.population_size
        population = np.apply_along_axis(task.repair, 1,
                                         self.migration_operator(task.dimension, self.np1, self.np2, population))
        population = np.apply_along_axis(task.repair, 1,
                                         self.adjusting_operator(task.iters, max_t, task.dimension, self.np1, self.np2,
                                                                 population, current_best))
        population_fitness, population = self.evaluate_and_sort(task, population)
        current_best = population[0]
        population[-self.keep:] = elite
        population_fitness, population = self.evaluate_and_sort(task, population)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {'current_best': current_best}



import numpy as np
from numpy.random import default_rng

__all__ = ['limit', 'limit_inverse', 'wang', 'rand', 'reflect']


def limit(x, lower, upper, **_kwargs):
    r"""Repair solution and put the solution in the random position inside of the bounds of problem.

    Args:
        x (numpy.ndarray): Solution to check and repair if needed.
        lower (numpy.ndarray): Lower bounds of search space.
        upper (numpy.ndarray): Upper bounds of search space.

    Returns:
        numpy.ndarray: Solution in search space.

    """
    return np.clip(x, lower, upper, out=x)


def limit_inverse(x, lower, upper, **_kwargs):
    r"""Repair solution and put the solution in the random position inside of the bounds of problem.

    Args:
        x (numpy.ndarray): Solution to check and repair if needed.
        lower (numpy.ndarray): Lower bounds of search space.
        upper (numpy.ndarray): Upper bounds of search space.

    Returns:
        numpy.ndarray: Solution in search space.

    """
    ir = np.where(x < lower)
    x[ir] = upper[ir]
    ir = np.where(x > upper)
    x[ir] = lower[ir]
    return x


def wang(x, lower, upper, **_kwargs):
    r"""Repair solution and put the solution in the random position inside of the bounds of problem.

    Args:
        x (numpy.ndarray): Solution to check and repair if needed.
        lower (numpy.ndarray): Lower bounds of search space.
        upper (numpy.ndarray): Upper bounds of search space.

    Returns:
        numpy.ndarray: Solution in search space.

    """
    ir = np.where(x < lower)
    x[ir] = np.amin([upper[ir], 2 * lower[ir] - x[ir]], axis=0)
    ir = np.where(x > upper)
    x[ir] = np.amax([lower[ir], 2 * upper[ir] - x[ir]], axis=0)
    return x


def rand(x, lower, upper, rng=None, **_kwargs):
    r"""Repair solution and put the solution in the random position inside of the bounds of problem.

    Args:
        x (numpy.ndarray): Solution to check and repair if needed.
        lower (numpy.ndarray): Lower bounds of search space.
        upper (numpy.ndarray): Upper bounds of search space.
        rng (numpy.random.Generator): Random generator.

    Returns:
        numpy.ndarray: Fixed solution.

    """
    rng = default_rng(rng)
    ir = np.where(x < lower)
    x[ir] = rng.uniform(lower[ir], upper[ir])
    ir = np.where(x > upper)
    x[ir] = rng.uniform(lower[ir], upper[ir])
    return x


def reflect(x, lower, upper, **_kwargs):
    r"""Repair solution and put the solution in search space with reflection of how much the solution violates a bound.

    Args:
        x (numpy.ndarray): Solution to be fixed.
        lower (numpy.ndarray): Lower bounds of search space.
        upper (numpy.ndarray): Upper bounds of search space.

    Returns:
        numpy.ndarray: Fix solution.

    """
    ir = np.where(x > upper)
    x[ir] = lower[ir] + x[ir] % (upper[ir] - lower[ir])
    ir = np.where(x < lower)
    x[ir] = lower[ir] + x[ir] % (upper[ir] - lower[ir])
    return x



# encoding=utf8

"""Styblinski Tang problem."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['StyblinskiTang']


class StyblinskiTang(Problem):
    r"""Implementation of Styblinski-Tang functions.

    Date: 2018

    Authors: Lucija Brezočnik

    License: MIT

    Function: **Styblinski-Tang function**

        :math:`f(\mathbf{x}) = \frac{1}{2} \sum_{i=1}^D \left(
        x_i^4 - 16x_i^2 + 5x_i \right)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-5, 5]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = -78.332`, at :math:`x^* = (-2.903534,...,-2.903534)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \frac{1}{2} \sum_{i=1}^D \left(
            x_i^4 - 16x_i^2 + 5x_i \right) $

        Equation:
            \begin{equation}f(\mathbf{x}) =
            \frac{1}{2} \sum_{i=1}^D \left( x_i^4 - 16x_i^2 + 5x_i \right) \end{equation}

        Domain:
            $-5 \leq x_i \leq 5$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    def __init__(self, dimension=4, lower=-5.0, upper=5.0, *args, **kwargs):
        r"""Initialize Styblinski Tang problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \frac{1}{2} \sum_{i=1}^D \left(
                x_i^4 - 16x_i^2 + 5x_i \right) $'''

    def _evaluate(self, x):
        return 0.5 * np.sum(x ** 4 - 16.0 * x ** 2 + 5.0 * x)



# encoding=utf8

"""Implementation of Ackley problem."""

import numpy as np

from niapy.problems.problem import Problem

__all__ = ['Ackley']


class Ackley(Problem):
    r"""Implementation of Ackley function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Ackley function**

        :math:`f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}\sum_{i=1}^D x_i^2}\right)
        - \exp\left(\frac{1}{D}\sum_{i=1}^D \cos(c\;x_i)\right) + a + \exp(1)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-32.768, 32.768]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(\textbf{x}^*) = 0`, at  :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}
            \sum_{i=1}^D x_i^2}\right) - \exp\left(\frac{1}{D}
            \sum_{i=1}^D cos(c\;x_i)\right) + a + \exp(1)$

        Equation:
            \begin{equation}f(\mathbf{x}) =
            -a\;\exp\left(-b \sqrt{\frac{1}{D} \sum_{i=1}^D x_i^2}\right) -
            \exp\left(\frac{1}{D} \sum_{i=1}^D \cos(c\;x_i)\right) +
            a + \exp(1) \end{equation}

        Domain:
            $-32.768 \leq x_i \leq 32.768$

    Reference:
        https://www.sfu.ca/~ssurjano/ackley.html

    """

    def __init__(self, dimension=4, lower=-32.768, upper=32.768, a=20.0, b=0.2, c=2 * np.pi, *args, **kwargs):
        r"""Initialize Ackley problem.

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.
            a (Optional[float]): a parameter.
            b (Optional[float]): b parameter.
            c (Optional[float]): c parameter.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)
        self.a = a
        self.b = b
        self.c = c

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}
                \sum_{i=1}^D x_i^2}\right) - \exp\left(\frac{1}{D}
                \sum_{i=1}^D \cos(c\;x_i)\right) + a + \exp(1)$'''

    def _evaluate(self, x):
        val1 = np.sum(np.square(x))
        val2 = np.sum(np.cos(self.c * x))

        temp1 = -self.b * np.sqrt(val1 / self.dimension)
        temp2 = val2 / self.dimension

        return -self.a * np.exp(temp1) - np.exp(temp2) + self.a + np.exp(1)



# encoding=utf8

"""Implementations of Michalewicz's function."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Michalewicz']


class Michalewicz(Problem):
    r"""Implementations of Michalewicz's functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **High Conditioned Elliptic Function**

        :math:`f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [0, \pi]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        at :math:`d = 2` :math:`f(\textbf{x}^*) = -1.8013` at :math:`\textbf{x}^* = (2.20, 1.57)`
        at :math:`d = 5` :math:`f(\textbf{x}^*) = -4.687658`
        at :math:`d = 10` :math:`f(\textbf{x}^*) = -9.66015`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m}$

        Equation:
            \begin{equation} f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m} \end{equation}

        Domain:
            $0 \leq x_i \leq \pi$

    Reference URL:
        https://www.sfu.ca/~ssurjano/michal.html

    """

    def __init__(self, dimension=4, lower=0.0, upper=np.pi, m=10, *args, **kwargs):
        r"""Initialize Michalewicz problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.
            m (float): Steepness of valleys and ridges. Recommended value is 10.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)
        self.m = m

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m}$'''

    def _evaluate(self, x):
        return -np.sum(np.sin(x) * np.sin((np.arange(1, self.dimension + 1) * x ** 2.0) / np.pi) ** (2.0 * self.m))



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.other')
logger.setLevel('INFO')

__all__ = ['NelderMeadMethod']


class NelderMeadMethod(Algorithm):
    r"""Implementation of Nelder Mead method or downhill simplex method or amoeba method.

    Algorithm:
        Nelder Mead Method

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        https://en.wikipedia.org/wiki/Nelder%E2%80%93Mead_method

    Attributes:
        Name (List[str]): list of strings representing algorithm name
        alpha (float): Reflection coefficient parameter
        gamma (float): Expansion coefficient parameter
        rho (float): Contraction coefficient parameter
        sigma (float): Shrink coefficient parameter

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['NelderMeadMethod', 'NMM']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information of algorithm.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""No info"""

    def __init__(self, population_size=None, alpha=0.1, gamma=0.3, rho=-0.2, sigma=-0.2, *args, **kwargs):
        """Initialize NelderMeadMethod.

        Args:
            population_size (Optional[int]): Number of individuals.
            alpha (Optional[float]): Reflection coefficient parameter
            gamma (Optional[float]): Expansion coefficient parameter
            rho (Optional[float]): Contraction coefficient parameter
            sigma (Optional[float]): Shrink coefficient parameter

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, initialization_function=kwargs.pop('initialization_function', self.init_pop), *args, **kwargs)
        self.alpha = alpha
        self.gamma = gamma
        self.rho = rho
        self.sigma = sigma

    def set_parameters(self, population_size=None, alpha=0.1, gamma=0.3, rho=-0.2, sigma=-0.2, **kwargs):
        r"""Set the arguments of an algorithm.

        Args:
            population_size (Optional[int]): Number of individuals.
            alpha (Optional[float]): Reflection coefficient parameter
            gamma (Optional[float]): Expansion coefficient parameter
            rho (Optional[float]): Contraction coefficient parameter
            sigma (Optional[float]): Shrink coefficient parameter

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, initialization_function=kwargs.pop('initialization_function', self.init_pop), **kwargs)
        self.alpha = alpha
        self.gamma = gamma
        self.rho = rho
        self.sigma = sigma

    def get_parameters(self):
        d = Algorithm.get_parameters(self)
        d.update({
            'alpha': self.alpha,
            'gamma': self.gamma,
            'rho': self.rho,
            'sigma': self.sigma
        })
        return d

    def init_pop(self, task, population_size, **_kwargs):
        r"""Init starting population.

        Args:
            population_size (int): Number of individuals in population.
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float]]:
                1. New initialized population.
                2. New initialized population fitness/function values.

        """
        population_size = task.dimension if population_size is None or population_size < task.dimension else population_size
        population = self.uniform(task.lower, task.upper, (population_size, task.dimension))
        population_fitness = np.apply_along_axis(task.eval, 1, population)
        return population, population_fitness

    def method(self, population, population_fitness, task):
        r"""Run the main function.

        Args:
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray[float]): Current population function/fitness values.
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float]]:
                1. New population.
                2. New population fitness/function values.

        """
        x0 = np.sum(population[:-1], axis=0) / (len(population) - 1)
        xr = x0 + self.alpha * (x0 - population[-1])
        rs = task.eval(xr)
        if population_fitness[0] >= rs < population_fitness[-2]:
            population[-1], population_fitness[-1] = xr, rs
            return population, population_fitness
        if rs < population_fitness[0]:
            xe = x0 + self.gamma * (x0 - population[-1])
            re = task.eval(xe)
            if re < rs:
                population[-1], population_fitness[-1] = xe, re
            else:
                population[-1], population_fitness[-1] = xr, rs
            return population, population_fitness
        xc = x0 + self.rho * (x0 - population[-1])
        rc = task.eval(xc)
        if rc < population_fitness[-1]:
            population[-1], population_fitness[-1] = xc, rc
            return population, population_fitness
        new_population = population[0] + self.sigma * (population[1:] - population[0])
        new_population_fitness = np.apply_along_axis(task.eval, 1, new_population)
        population[1:], population_fitness[1:] = new_population, new_population_fitness
        return population, population_fitness

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core iteration function of NelderMeadMethod algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current populations fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best function/fitness value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Additional arguments.

        """
        sorted_indices = np.argsort(population_fitness)
        population, population_fitness = population[sorted_indices], population_fitness[sorted_indices]
        population, population_fitness = self.method(population, population_fitness, task)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {}



# coding=utf-8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

__all__ = ['BeesAlgorithm']

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')


class BeesAlgorithm(Algorithm):
    r"""Implementation of Bees algorithm.

    Algorithm:
        The Bees algorithm

    Date:
        2019

    Authors:
        Rok Potočnik

    License:
        MIT

    Reference paper:
        DT Pham, A Ghanbarzadeh, E Koc, S Otri, S Rahim, and M Zaidi. The bees algorithm-a novel tool for complex optimisation problems. In Proceedings of the 2nd Virtual International Conference on Intelligent Production Machines and Systems (IPROMS 2006), pages 454–459, 2006

    Attributes:
        population_size (Optional[int]): Number of scout bees parameter.
        m (Optional[int]): Number of sites selected out of n visited sites parameter.
        e (Optional[int]): Number of best sites out of m selected sites parameter.
        nep (Optional[int]): Number of bees recruited for best e sites parameter.
        nsp (Optional[int]): Number of bees recruited for the other selected sites parameter.
        ngh (Optional[float]): Initial size of patches parameter.

    See Also:
        * :func:`niapy.algorithms.Algorithm.set_parameters`

    """

    Name = ['BeesAlgorithm', 'BEA']

    @staticmethod
    def info():
        r"""Get information about algorithm.

        Returns:
            str: Algorithm information

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""DT Pham, A Ghanbarzadeh, E Koc, S Otri, S Rahim, and M Zaidi. The bees algorithm-a novel tool for complex optimisation problems. In Proceedings of the 2nd Virtual International Conference on Intelligent Production Machines and Systems (IPROMS 2006), pages 454–459, 2006"""

    def __init__(self, population_size=40, m=5, e=4, ngh=1, nep=4, nsp=2, *args, **kwargs):
        """Initialize BeesAlgorithm.

        Args:
            population_size (Optional[int]): Number of scout bees parameter.
            m (Optional[int]): Number of sites selected out of n visited sites parameter.
            e (Optional[int]): Number of best sites out of m selected sites parameter.
            nep (Optional[int]): Number of bees recruited for best e sites parameter.
            nsp (Optional[int]): Number of bees recruited for the other selected sites parameter.
            ngh (Optional[float]): Initial size of patches parameter.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.m = m
        self.e = e
        self.ngh = ngh
        self.nep = nep
        self.nsp = nsp

    def set_parameters(self, population_size=40, m=5, e=4, ngh=1, nep=4, nsp=2, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Number of scout bees parameter.
            m (Optional[int]): Number of sites selected out of n visited sites parameter.
            e (Optional[int]): Number of best sites out of m selected sites parameter.
            nep (Optional[int]): Number of bees recruited for best e sites parameter.
            nsp (Optional[int]): Number of bees recruited for the other selected sites parameter.
            ngh (Optional[float]): Initial size of patches parameter.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.m = m
        self.e = e
        self.ngh = ngh
        self.nep = nep
        self.nsp = nsp

    def get_parameters(self):
        r"""Get parameters of the algorithm.

        Returns:
            Dict[str, Any]: Algorithm Parameters.

        """
        d = super().get_parameters()
        d.update({
            'm': self.m,
            'e': self.e,
            'ngh': self.ngh,
            'nep': self.nep,
            'nsp': self.nsp
        })
        return d

    def bee_dance(self, x, task, ngh):
        r"""Bees Dance. Search for new positions.

        Args:
            x (numpy.ndarray): One individual from the population.
            task (Task): Optimization task.
            ngh (float): A small value for patch search.

        Returns:
            Tuple[numpy.ndarray, float]:
                1. New individual.
                2. New individual fitness/function values.

        """
        ind = self.integers(task.dimension)
        y = x.copy()
        y[ind] = x[ind] + self.uniform(-ngh, ngh)
        y = task.repair(y)
        y_fitness = task.eval(y)
        return y, y_fitness

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        population, population_fitness, _ = super().init_population(task)

        sorted_indices = np.argsort(population_fitness)
        population_fitness = population_fitness[sorted_indices]
        population = population[sorted_indices, :]

        return population, population_fitness, {'ngh': self.ngh}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Forest Optimization Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray[float]): Current population.
            population_fitness (numpy.ndarray[float]): Current population function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution.
                4. New global best fitness/objective value.
                5. Additional arguments:
                    * ngh (float): A small value used for patches.

        """
        ngh = params.pop('ngh')

        for ies in range(self.e):
            best_bee_pos = None
            best_bee_cost = np.inf
            for ieb in range(self.nep):
                new_bee_pos, new_bee_cost = self.bee_dance(population[ies, :], task, ngh)
                if new_bee_cost < best_bee_cost:
                    best_bee_cost = new_bee_cost
                    best_bee_pos = new_bee_pos
            if best_bee_cost < population_fitness[ies]:
                population[ies, :] = best_bee_pos
                population_fitness[ies] = best_bee_cost
        for ies in range(self.e, self.m):
            best_bee_pos = None
            best_bee_cost = np.inf
            for ieb in range(self.nsp):
                new_bee_pos, new_bee_cost = self.bee_dance(population[ies, :], task, ngh)
                if new_bee_cost < best_bee_cost:
                    best_bee_cost = new_bee_cost
                    best_bee_pos = new_bee_pos
            if best_bee_cost < population_fitness[ies]:
                population[ies, :] = best_bee_pos
                population_fitness[ies] = best_bee_cost
        for ies in range(self.m, self.population_size):
            population[ies, :] = self.uniform(task.lower, task.upper, task.dimension)
            population_fitness[ies] = task.eval(population[ies, :])
        sorted_indices = np.argsort(population_fitness)
        population_fitness = population_fitness[sorted_indices]
        population = population[sorted_indices, :]
        ngh = ngh * 0.95
        return population, population_fitness, population[0].copy(), population_fitness[0], {'ngh': ngh}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.modified')
logger.setLevel('INFO')

__all__ = ['ParameterFreeBatAlgorithm']


class ParameterFreeBatAlgorithm(Algorithm):
    r"""Implementation of Parameter-free Bat algorithm.

    Algorithm:
        Parameter-free Bat algorithm

    Date:
        2020

    Authors:
        Iztok Fister Jr.
        This implementation is based on the implementation of basic BA from niapy

    License:
        MIT

    Reference paper:
        Iztok Fister Jr., Iztok Fister, Xin-She Yang. Towards the development of a parameter-free bat algorithm . In: FISTER Jr., Iztok (Ed.), BRODNIK, Andrej (Ed.). StuCoSReC : proceedings of the 2015 2nd Student Computer Science Research Conference. Koper: University of Primorska, 2015, pp. 31-34.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['ParameterFreeBatAlgorithm', 'PLBA']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Iztok Fister Jr., Iztok Fister, Xin-She Yang. Towards the development of a parameter-free bat algorithm . In: FISTER, Iztok (Ed.), BRODNIK, Andrej (Ed.). StuCoSReC : proceedings of the 2015 2nd Student Computer Science Research Conference. Koper: University of Primorska, 2015, pp. 31-34."""

    def __init__(self, *args, **kwargs):
        """Initialize ParameterFreeBatAlgorithm."""
        super().__init__(80, *args, **kwargs)
        self.loudness = 0.9
        self.pulse_rate = 0.1

    def set_parameters(self, **kwargs):
        r"""Set the parameters of the algorithm.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=80, **kwargs)
        self.loudness = 0.9
        self.pulse_rate = 0.1

    def init_population(self, task):
        r"""Initialize the initial population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * velocities (numpy.ndarray[float]): Velocities

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        population, fitness, d = Algorithm.init_population(self, task)
        velocities = np.zeros((self.population_size, task.dimension))
        d.update({'velocities': velocities})
        return population, fitness, d

    def local_search(self, best, task, **_kwargs):
        r"""Improve the best solution according to the Yang (2010).

        Args:
            best (numpy.ndarray): Global best individual.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: New solution based on global best individual.

        """
        return task.repair(best + 0.001 * self.normal(0, 1, task.dimension))

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Parameter-free Bat Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population
            population_fitness (numpy.ndarray[float]): Current population fitness/function values
            best_x (numpy.ndarray): Current best individual
            best_fitness(float): Current best individual function/fitness value
            params (Dict[str, Any]): Additional algorithm arguments

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. New global best solution
                4. New global best fitness/objective value
                5. Additional arguments:
                    * velocities (numpy.ndarray): Velocities

        """
        velocities = params.pop('velocities')
        upper, lower = task.upper, task.lower
        for i in range(self.population_size):
            frequency = ((upper[0] - lower[0]) / float(self.population_size)) * self.normal(0, 1)
            velocities[i] += (population[i] - best_x) * frequency
            if self.random() > self.pulse_rate:
                solution = self.local_search(best=best_x, task=task, i=i, Sol=population)
            else:
                solution = task.repair(population[i] + velocities[i], rng=self.rng)
            new_fitness = task.eval(solution)
            if (new_fitness <= population_fitness[i]) and (self.random() < self.loudness):
                population[i], population_fitness[i] = solution, new_fitness
            if new_fitness <= best_fitness:
                best_x, best_fitness = solution.copy(), new_fitness
        return population, population_fitness, best_x, best_fitness, {'velocities': velocities}



# encoding=utf8

"""Implementations of Perm function."""
import numpy as np

from niapy.problems.problem import Problem

__all__ = ['Perm']


class Perm(Problem):
    r"""Implementations of Perm functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Perm Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-D, D]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (1, \frac{1}{2}, \cdots , \frac{1}{i} , \cdots , \frac{1}{D})`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2 \end{equation}

        Domain:
            $-D \leq x_i \leq D$

    Reference:
        https://www.sfu.ca/~ssurjano/perm0db.html


    """

    def __init__(self, dimension=4, beta=0.5, *args, **kwargs):
        r"""Initialize Perm problem.

        Args:
            dimension (Optional[int]): Dimension of the problem.
            beta (Optional[float]): Beta parameter.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, -dimension, dimension, *args, **kwargs)
        self.beta = beta

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2$'''

    def _evaluate(self, x):
        ii = np.arange(1, self.dimension + 1)
        jj = np.tile(ii, (self.dimension, 1))
        x_matrix = np.tile(x, (self.dimension, 1))
        inner = np.sum((jj + self.beta) * (np.power(x_matrix, ii) - np.power(1.0 / jj, ii)), axis=0)
        return np.sum(inner ** 2)



# encoding=utf8

"""Rosenbrock problem."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Rosenbrock']


class Rosenbrock(Problem):
    r"""Implementation of Rosenbrock problem.

    Date: 2018

    Authors: Iztok Fister Jr. and Lucija Brezočnik

    License: MIT

    Function: **Rosenbrock function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^{D-1} \left (100 (x_{i+1} - x_i^2)^2 + (x_i - 1)^2 \right)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-30, 30]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (1,...,1)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^{D-1} (100 (x_{i+1} - x_i^2)^2 + (x_i - 1)^2)$

        Equation:
            \begin{equation}
            f(\mathbf{x}) = \sum_{i=1}^{D-1} (100 (x_{i+1} - x_i^2)^2 + (x_i - 1)^2)
            \end{equation}

        Domain:
            $-30 \leq x_i \leq 30$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    def __init__(self, dimension=4, lower=-30.0, upper=30.0, *args, **kwargs):
        r"""Initialize Rosenbrock problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^{D-1} (100 (x_{i+1} - x_i^2)^2 + (x_i - 1)^2)$'''

    def _evaluate(self, x):
        return np.sum(100.0 * (x[1:] - x[:-1] ** 2.0) ** 2.0 + (1 - x[:-1]) ** 2.0, axis=0)



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

__all__ = ['ForestOptimizationAlgorithm']

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')


class ForestOptimizationAlgorithm(Algorithm):
    r"""Implementation of Forest Optimization Algorithm.

    Algorithm:
        Forest Optimization Algorithm

    Date:
        2019

    Authors:
        Luka Pečnik

    License:
        MIT

    Reference paper:
        Manizheh Ghaemi, Mohammad-Reza Feizi-Derakhshi, Forest Optimization Algorithm, Expert Systems with Applications, Volume 41, Issue 15, 2014, Pages 6676-6687, ISSN 0957-4174, https://doi.org/10.1016/j.eswa.2014.05.009.

    References URL:
        Implementation is based on the following MATLAB code: https://github.com/cominsys/FOA

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        lifetime (int): Life time of trees parameter.
        area_limit (int): Area limit parameter.
        local_seeding_changes (int): Local seeding changes parameter.
        global_seeding_changes (int): Global seeding changes parameter.
        transfer_rate (float): Transfer rate parameter.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['ForestOptimizationAlgorithm', 'FOA']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Manizheh Ghaemi, Mohammad-Reza Feizi-Derakhshi, Forest Optimization Algorithm, Expert Systems with Applications, Volume 41, Issue 15, 2014, Pages 6676-6687, ISSN 0957-4174, https://doi.org/10.1016/j.eswa.2014.05.009."""

    def __init__(self, population_size=10, lifetime=3, area_limit=10, local_seeding_changes=1, global_seeding_changes=1,
                 transfer_rate=0.3, *args, **kwargs):
        """Initialize ForestOptimizationAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            lifetime (Optional[int]): Life time parameter.
            area_limit (Optional[int]): Area limit parameter.
            local_seeding_changes (Optional[int]): Local seeding changes parameter.
            global_seeding_changes (Optional[int]): Global seeding changes parameter.
            transfer_rate (Optional[float]): Transfer rate parameter.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.lifetime = lifetime
        self.area_limit = area_limit
        self.local_seeding_changes = local_seeding_changes
        self.global_seeding_changes = global_seeding_changes
        self.transfer_rate = transfer_rate
        self.dx = None

    def set_parameters(self, population_size=10, lifetime=3, area_limit=10, local_seeding_changes=1,
                       global_seeding_changes=1, transfer_rate=0.3, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            lifetime (Optional[int]): Life time parameter.
            area_limit (Optional[int]): Area limit parameter.
            local_seeding_changes (Optional[int]): Local seeding changes parameter.
            global_seeding_changes (Optional[int]): Global seeding changes parameter.
            transfer_rate (Optional[float]): Transfer rate parameter.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.lifetime = lifetime
        self.area_limit = area_limit
        self.local_seeding_changes = local_seeding_changes
        self.global_seeding_changes = global_seeding_changes
        self.transfer_rate = transfer_rate
        self.dx = None

    def get_parameters(self):
        r"""Get parameters values of the algorithm.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = Algorithm.get_parameters(self)
        d.update({
            'lifetime': self.lifetime,
            'area_limit': self.area_limit,
            'local_seeding_changes': self.local_seeding_changes,
            'global_seeding_changes': self.global_seeding_changes,
            'transfer_rate': self.transfer_rate
        })
        return d

    def local_seeding(self, task, trees):
        r"""Local optimum search stage.

        Args:
            task (Task): Optimization task.
            trees (numpy.ndarray): Zero age trees for local seeding.

        Returns:
            numpy.ndarray: Resulting zero age trees.

        """
        seeds = np.repeat(trees, self.local_seeding_changes, axis=0)
        for i in range(seeds.shape[0]):
            indices = self.rng.choice(task.dimension, self.local_seeding_changes, replace=False)
            seeds[i, indices] += self.uniform(-self.dx[indices], self.dx[indices])
            seeds[i] = task.repair(seeds[i], rng=self.rng)
        return seeds

    def global_seeding(self, task, candidates, size):
        r"""Global optimum search stage that should prevent getting stuck in a local optimum.

        Args:
            task (Task): Optimization task.
            candidates (numpy.ndarray): Candidate population for global seeding.
            size (int): Number of trees to produce.

        Returns:
            numpy.ndarray: Resulting trees.

        """
        seeds = candidates[self.rng.choice(len(candidates), size, replace=False)]
        for i in range(seeds.shape[0]):
            indices = self.rng.choice(task.dimension, self.global_seeding_changes, replace=False)
            seeds[i, indices] = self.uniform(task.lower[indices], task.upper[indices])
        return seeds

    def remove_lifetime_exceeded(self, trees, age):
        r"""Remove dead trees.

        Args:
            trees (numpy.ndarray): Population to test.
            age (numpy.ndarray[int32]): Age of trees.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray[int32]]:
                1. Alive trees.
                2. New candidate population.
                3. Age of trees.

        """
        life_time_exceeded = np.where(age > self.lifetime)
        candidates = trees[life_time_exceeded]
        trees = np.delete(trees, life_time_exceeded, axis=0)
        age = np.delete(age, life_time_exceeded, axis=0)
        return trees, candidates, age

    def survival_of_the_fittest(self, task, trees, candidates, age):
        r"""Evaluate and filter current population.

        Args:
            task (Task): Optimization task.
            trees (numpy.ndarray): Population to evaluate.
            candidates (numpy.ndarray): Candidate population array to be updated.
            age (numpy.ndarray[int32]): Age of trees.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray[float], numpy.ndarray[int32]]:
                1. Trees sorted by fitness value.
                2. Updated candidate population.
                3. Population fitness values.
                4. Age of trees

        """
        evaluations = np.apply_along_axis(task.eval, 1, trees)
        ei = evaluations.argsort()
        candidates = np.append(candidates, trees[ei[self.area_limit:]], axis=0)
        trees = trees[ei[:self.area_limit]]
        age = age[ei[:self.area_limit]]
        evaluations = evaluations[ei[:self.area_limit]]
        return trees, candidates, evaluations, age

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * age (numpy.ndarray[int32]): Age of trees.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        trees, fitness, _ = Algorithm.init_population(self, task)
        age = np.zeros(self.population_size, dtype=np.int32)
        self.dx = np.absolute(task.upper) / 5.0
        return trees, fitness, {'age': age}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Forest Optimization Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray[float]): Current population function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * age (numpy.ndarray[int32]): Age of trees.

        """
        age = params.pop('age')

        zero_age_trees = population[age == 0]
        local_seeds = self.local_seeding(task, zero_age_trees)
        age += 1
        population, candidate_population, age = self.remove_lifetime_exceeded(population, age)
        population = np.append(population, local_seeds, axis=0)
        age = np.append(age, np.zeros(len(local_seeds), dtype=np.int32))
        population, candidate_population, population_fitness, age = self.survival_of_the_fittest(task, population, candidate_population, age)
        gsn = int(self.transfer_rate * len(candidate_population))
        if gsn > 0:
            global_seeds = self.global_seeding(task, candidate_population, gsn)
            population = np.append(population, global_seeds, axis=0)
            age = np.append(age, np.zeros(len(global_seeds), dtype=np.int32))
            global_seeds_fitness = np.apply_along_axis(task.eval, 1, global_seeds)
            population_fitness = np.append(population_fitness, global_seeds_fitness)
        ib = np.argmin(population_fitness)
        age[ib] = 0
        if population_fitness[ib] < best_fitness:
            best_x, best_fitness = population[ib].copy(), population_fitness[ib]
        return population, population_fitness, best_x, best_fitness, {'age': age}



# encoding=utf8

"""Implementation of Happy Cat function."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['HappyCat']


class HappyCat(Problem):
    r"""Implementation of Happy cat function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Happy cat function**

        :math:`f(\mathbf{x}) = {\left |\sum_{i = 1}^D {x_i}^2 - D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 + \sum_{i = 1}^D x_i) / D + 0.5`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (-1,...,-1)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = {\left|\sum_{i = 1}^D {x_i}^2 -
            D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
            \sum_{i = 1}^D x_i) / D + 0.5$

        Equation:
            \begin{equation} f(\mathbf{x}) = {\left| \sum_{i = 1}^D {x_i}^2 -
            D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
            \sum_{i = 1}^D x_i) / D + 0.5 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference: http://bee22.com/manual/tf_images/Liang%20CEC2014.pdf &
    Beyer, H. G., & Finck, S. (2012). HappyCat - A Simple Function Class Where Well-Known Direct Search Algorithms Do Fail.
    In International Conference on Parallel Problem Solving from Nature (pp. 367-376). Springer, Berlin, Heidelberg.

    """

    def __init__(self, dimension=4, lower=-100.0, upper=100.0, alpha=0.25, *args, **kwargs):
        r"""Initialize Happy cat problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)
        self.alpha = alpha

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = {\left|\sum_{i = 1}^D {x_i}^2 -
                D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
                \sum_{i = 1}^D x_i) / D + 0.5$'''

    def _evaluate(self, x):
        val1 = np.sum(np.abs(x * x - self.dimension) ** self.alpha)
        val2 = np.sum((0.5 * x * x + x) / self.dimension)
        return val1 + val2 + 0.5



"""Module with implementations of optimization problems."""

from niapy.problems.ackley import Ackley
from niapy.problems.alpine import Alpine1, Alpine2
from niapy.problems.problem import Problem
from niapy.problems.bent_cigar import BentCigar
from niapy.problems.chung_reynolds import ChungReynolds
from niapy.problems.cosine_mixture import CosineMixture
from niapy.problems.csendes import Csendes
from niapy.problems.discus import Discus
from niapy.problems.dixon_price import DixonPrice
from niapy.problems.elliptic import Elliptic
from niapy.problems.griewank import Griewank, ExpandedGriewankPlusRosenbrock
from niapy.problems.happy_cat import HappyCat
from niapy.problems.hgbat import HGBat
from niapy.problems.katsuura import Katsuura
from niapy.problems.levy import Levy
from niapy.problems.michalewicz import Michalewicz
from niapy.problems.perm import Perm
from niapy.problems.pinter import Pinter
from niapy.problems.powell import Powell
from niapy.problems.qing import Qing
from niapy.problems.quintic import Quintic
from niapy.problems.rastrigin import Rastrigin
from niapy.problems.ridge import Ridge
from niapy.problems.rosenbrock import Rosenbrock
from niapy.problems.salomon import Salomon
from niapy.problems.schaffer import SchafferN2, SchafferN4, ExpandedSchaffer
from niapy.problems.schumer_steiglitz import SchumerSteiglitz
from niapy.problems.schwefel import Schwefel, Schwefel221, Schwefel222, ModifiedSchwefel
from niapy.problems.sphere import Sphere, Sphere2, Sphere3
from niapy.problems.step import Step, Step2, Step3
from niapy.problems.stepint import Stepint
from niapy.problems.styblinski_tang import StyblinskiTang
from niapy.problems.sum_squares import SumSquares
from niapy.problems.trid import Trid
from niapy.problems.weierstrass import Weierstrass
from niapy.problems.whitley import Whitley
from niapy.problems.zakharov import Zakharov

__all__ = [
    'Problem',
    'Rastrigin',
    'Rosenbrock',
    'Griewank',
    'ExpandedGriewankPlusRosenbrock',
    'Sphere',
    'Ackley',
    'Schwefel',
    'Schwefel221',
    'Schwefel222',
    'ModifiedSchwefel',
    'Whitley',
    'Alpine1',
    'Alpine2',
    'HappyCat',
    'Ridge',
    'ChungReynolds',
    'Csendes',
    'Pinter',
    'Qing',
    'Quintic',
    'Salomon',
    'SchumerSteiglitz',
    'Step',
    'Step2',
    'Step3',
    'Stepint',
    'SumSquares',
    'StyblinskiTang',
    'BentCigar',
    'Weierstrass',
    'HGBat',
    'Katsuura',
    'Elliptic',
    'Discus',
    'Michalewicz',
    'Levy',
    'Sphere2',
    'Sphere3',
    'Trid',
    'Perm',
    'Zakharov',
    'DixonPrice',
    'Powell',
    'CosineMixture',
    'ExpandedSchaffer',
    'SchafferN2',
    'SchafferN4'
]



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm
from niapy.util import levy_flight

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['CuckooSearch']


class CuckooSearch(Algorithm):
    r"""Implementation of Cuckoo behaviour and levy flights.

    Algorithm:
        Cuckoo Search

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference:
        Yang, Xin-She, and Suash Deb. "Cuckoo search via Lévy flights."
        Nature & Biologically Inspired Computing, 2009. NaBIC 2009. World Congress on. IEEE, 2009.

    Attributes:
        Name (List[str]): list of strings representing algorithm names.
        pa (float): Probability of a nest being abandoned.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['CuckooSearch', 'CS']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Yang, Xin-She, and Suash Deb. "Cuckoo search via Lévy flights."
        Nature & Biologically Inspired Computing, 2009. NaBIC 2009. World Congress on. IEEE, 2009."""

    def __init__(self, population_size=25, pa=0.25, *args, **kwargs):
        r"""Initialize CuckooSearch.

        Args:
            population_size (int): Population size.
            pa (float): Probability of a nest being abandoned.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.pa = pa

    def set_parameters(self, population_size=50, pa=0.2, **kwargs):
        r"""Set the arguments of an algorithm.

        Args:
            population_size (int): Population size.
            pa (float): Probability of a nest being abandoned.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.pa = pa

    def get_parameters(self):
        """Get parameters of the algorithm."""
        d = super().get_parameters()
        d.update({
            'pa': self.pa,
        })
        return d

    def get_cuckoos(self, population, best_x, task):
        step_size = levy_flight(self.rng, size=population.shape) * (population - best_x)
        new_population = population + step_size * self.standard_normal(population.shape)
        return task.repair(new_population, rng=self.rng)

    def empty_nests(self, population, task):
        abandoned = self.random(population.shape) > self.pa
        i = self.rng.permutation(self.population_size)
        j = self.rng.permutation(self.population_size)
        step_size = self.random() * (population[i] - population[j])
        return task.repair(population + step_size * abandoned, rng=self.rng)

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of CuckooSearch algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current populations fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual function/fitness values.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. Initialized population.
                2. Initialized populations fitness/function values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments.

        """
        new_nests = self.get_cuckoos(population, best_x, task)
        new_fitness = np.apply_along_axis(task.eval, 1, new_nests)

        replace = new_fitness < population_fitness
        population[replace] = new_nests[replace]
        population_fitness[replace] = new_fitness[replace]
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)

        new_nests = self.empty_nests(population, task)
        new_fitness = np.apply_along_axis(task.eval, 1, new_nests)

        replace = new_fitness < population_fitness
        population[replace] = new_nests[replace]
        population_fitness[replace] = new_fitness[replace]
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)

        return population, population_fitness, best_x, best_fitness, {}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm
from niapy.util.distances import euclidean

__all__ = ['FireflyAlgorithm']

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')


class FireflyAlgorithm(Algorithm):
    r"""Implementation of Firefly algorithm.

    Algorithm:
        Firefly algorithm

    Date:
        2016

    Authors:
        Iztok Fister Jr, Iztok Fister and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Fister, I., Fister Jr, I., Yang, X. S., & Brest, J. (2013).
        A comprehensive review of firefly algorithms. Swarm and Evolutionary Computation, 13, 34-46.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        alpha (float): Randomness strength.
        beta0 (float): Attractiveness constant.
        gamma (float): Absorption coefficient.
        theta (float): Randomness reduction factor.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['FireflyAlgorithm', 'FA']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Fister, I., Fister Jr, I., Yang, X. S., & Brest, J. (2013). A comprehensive review of firefly algorithms. Swarm and Evolutionary Computation, 13, 34-46."""

    def __init__(self, population_size=20, alpha=1, beta0=1, gamma=0.01, theta=0.97, *args, **kwargs):
        """Initialize FireflyAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[float]): Randomness strength 0--1 (highly random).
            beta0 (Optional[float]): Attractiveness constant.
            gamma (Optional[float]): Absorption coefficient.
            theta (Optional[float]): Randomness reduction factor.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.alpha = alpha
        self.beta0 = beta0
        self.gamma = gamma
        self.theta = theta

    def set_parameters(self, population_size=20, alpha=1, beta0=1, gamma=0.01, theta=0.97, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[float]): Randomness strength 0--1 (highly random).
            beta0 (Optional[float]): Attractiveness constant.
            gamma (Optional[float]): Absorption coefficient.
            theta (Optional[float]): Randomness reduction factor.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.alpha = alpha
        self.beta0 = beta0
        self.gamma = gamma
        self.theta = theta

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * alpha (float): Randomness strength.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        fireflies, intensity, _ = super().init_population(task)
        return fireflies, intensity, {'alpha': self.alpha}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Firefly Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Additional arguments:
                    * alpha (float): Randomness strength.

        See Also:
            * :func:`niapy.algorithms.basic.FireflyAlgorithm.move_ffa`

        """
        alpha = params.pop('alpha') * self.theta

        for i in range(self.population_size):
            for j in range(self.population_size):
                if population_fitness[i] >= population_fitness[j]:
                    r = euclidean(population[i], population[j])
                    beta = self.beta0 * np.exp(-self.gamma * r ** 2)
                    steps = alpha * (self.random(task.dimension) - 0.5) * task.range
                    population[i] += beta * (population[j] - population[i]) + steps
                    population[i] = task.repair(population[i])
                    population_fitness[i] = task.eval(population[i])
                    best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)

        return population, population_fitness, best_x, best_fitness, {'alpha': alpha}
